@@ -92,10 +92,10 @@ Esys_EventSequenceComplete(
         r = Esys_EventSequenceComplete_Finish(esysContext, results);
         /* This is just debug information about the reattempt to finish the
            command */
-        if ((r & ~TSS2_RC_LAYER_MASK) == TSS2_BASE_RC_TRY_AGAIN)
+        if (base_rc(r) == TSS2_BASE_RC_TRY_AGAIN)
             LOG_DEBUG("A layer below returned TRY_AGAIN: %" PRIx32
                       " => resubmitting command", r);
-    } while ((r & ~TSS2_RC_LAYER_MASK) == TSS2_BASE_RC_TRY_AGAIN);
+    } while (base_rc(r) == TSS2_BASE_RC_TRY_AGAIN);
 
     /* Restore the timeout value to the original value */
     esysContext->timeout = timeouttmp;
@@ -189,11 +189,18 @@ Esys_EventSequenceComplete_Async(
     /* Calculate the cpHash Values */
     r = init_session_tab(esysContext, shandle1, shandle2, shandle3);
     return_state_if_error(r, _ESYS_STATE_INIT, "Initialize session resources");
-    iesys_compute_session_value(esysContext->session_tab[0],
+    if (pcrHandleNode != NULL)
+        iesys_compute_session_value(esysContext->session_tab[0],
                 &pcrHandleNode->rsrc.name, &pcrHandleNode->auth);
-    iesys_compute_session_value(esysContext->session_tab[1],
-                sequenceHandleNode ? &sequenceHandleNode->rsrc.name : NULL,
-                sequenceHandleNode ? &sequenceHandleNode->auth : NULL);
+    else
+        iesys_compute_session_value(esysContext->session_tab[0], NULL, NULL);
+
+    if (sequenceHandleNode != NULL)
+        iesys_compute_session_value(esysContext->session_tab[1],
+                &sequenceHandleNode->rsrc.name, &sequenceHandleNode->auth);
+    else
+        iesys_compute_session_value(esysContext->session_tab[1], NULL, NULL);
+
     iesys_compute_session_value(esysContext->session_tab[2], NULL, NULL);
 
     /* Generate the auth values and set them in the SAPI command buffer */
@@ -278,7 +285,7 @@ Esys_EventSequenceComplete_Finish(
 
     /*Receive the TPM response and handle resubmissions if necessary. */
     r = Tss2_Sys_ExecuteFinish(esysContext->sys, esysContext->timeout);
-    if ((r & ~TSS2_RC_LAYER_MASK) == TSS2_BASE_RC_TRY_AGAIN) {
+    if (base_rc(r) == TSS2_BASE_RC_TRY_AGAIN) {
         LOG_DEBUG("A layer below returned TRY_AGAIN: %" PRIx32, r);
         esysContext->state = _ESYS_STATE_SENT;
         goto error_cleanup;
