@@ -59,7 +59,7 @@ iesys_bn2binpad(const BIGNUM *bn, unsigned char *bin, int bin_length)
 }
 
 /** Context to hold temporary values for iesys_crypto */
-typedef struct _IESYS_CRYPTO_CONTEXT {
+typedef struct ESYS_CRYPTO_CONTEXT_BLOB {
     enum {
         IESYS_CRYPTOSSL_TYPE_HASH = 1,
         IESYS_CRYPTOSSL_TYPE_HMAC,
@@ -179,9 +179,12 @@ iesys_cryptossl_context_set_hash_md(IESYS_CRYPTOSSL_CONTEXT *ctx, TPM2_ALG_ID ha
  * @retval TSS2_ESYS_RC_GENERAL_FAILURE for errors of the crypto library.
  */
 TSS2_RC
-iesys_cryptossl_hash_start(IESYS_CRYPTO_CONTEXT_BLOB ** context,
-                           TPM2_ALG_ID hashAlg)
+iesys_cryptossl_hash_start(ESYS_CRYPTO_CONTEXT_BLOB ** context,
+                           TPM2_ALG_ID hashAlg,
+                           void *userdata)
 {
+    UNUSED(userdata);
+
     TSS2_RC r = TSS2_RC_SUCCESS;
     LOG_TRACE("call: context=%p hashAlg=%"PRIu16, context, hashAlg);
     return_if_null(context, "Context is NULL", TSS2_ESYS_RC_BAD_REFERENCE);
@@ -210,7 +213,7 @@ iesys_cryptossl_hash_start(IESYS_CRYPTO_CONTEXT_BLOB ** context,
         goto_error(r, TSS2_ESYS_RC_GENERAL_FAILURE, "Errror EVP_DigestInit", cleanup);
     }
 
-    *context = (IESYS_CRYPTO_CONTEXT_BLOB *) mycontext;
+    *context = (ESYS_CRYPTO_CONTEXT_BLOB *) mycontext;
 
     return TSS2_RC_SUCCESS;
 
@@ -231,9 +234,12 @@ iesys_cryptossl_hash_start(IESYS_CRYPTO_CONTEXT_BLOB ** context,
  * @retval TSS2_ESYS_RC_BAD_REFERENCE for invalid parameters.
  */
 TSS2_RC
-iesys_cryptossl_hash_update(IESYS_CRYPTO_CONTEXT_BLOB * context,
-                            const uint8_t * buffer, size_t size)
+iesys_cryptossl_hash_update(ESYS_CRYPTO_CONTEXT_BLOB * context,
+                            const uint8_t * buffer, size_t size,
+                            void *userdata)
 {
+    UNUSED(userdata);
+
     LOG_TRACE("called for context %p, buffer %p and size %zd", context, buffer,
               size);
     if (context == NULL || buffer == NULL) {
@@ -255,27 +261,6 @@ iesys_cryptossl_hash_update(IESYS_CRYPTO_CONTEXT_BLOB * context,
     return TSS2_RC_SUCCESS;
 }
 
-/** Update the digest value of a digest object from a TPM2B object.
- *
- * The context of a digest object will be updated according to the hash
- * algorithm of the context.
- * @param[in,out] context The context of the digest object which will be updated.
- * @param[in] b The TPM2B object for the update.
- * @retval TSS2_RC_SUCCESS on success.
- * @retval TSS2_ESYS_RC_BAD_REFERENCE for invalid parameters.
- */
-TSS2_RC
-iesys_cryptossl_hash_update2b(IESYS_CRYPTO_CONTEXT_BLOB * context, TPM2B * b)
-{
-    LOG_TRACE("called for context-pointer %p and 2b-pointer %p", context, b);
-    if (context == NULL || b == NULL) {
-        LOG_ERROR("Null-Pointer passed");
-        return TSS2_ESYS_RC_BAD_REFERENCE;
-    }
-    TSS2_RC ret = iesys_cryptossl_hash_update(context, &b->buffer[0], b->size);
-    return ret;
-}
-
 /** Get the digest value of a digest object and close the context.
  *
  * The digest value will written to a passed buffer and the resources of the
@@ -288,9 +273,12 @@ iesys_cryptossl_hash_update2b(IESYS_CRYPTO_CONTEXT_BLOB * context, TPM2B * b)
  * @retval TSS2_ESYS_RC_GENERAL_FAILURE for errors of the crypto library.
  */
 TSS2_RC
-iesys_cryptossl_hash_finish(IESYS_CRYPTO_CONTEXT_BLOB ** context,
-                            uint8_t * buffer, size_t * size)
+iesys_cryptossl_hash_finish(ESYS_CRYPTO_CONTEXT_BLOB ** context,
+                            uint8_t * buffer, size_t * size,
+                            void *userdata)
 {
+    UNUSED(userdata);
+
     unsigned int digest_size = 0;
 
     LOG_TRACE("called for context-pointer %p, buffer %p and size-pointer %p",
@@ -332,8 +320,12 @@ iesys_cryptossl_hash_finish(IESYS_CRYPTO_CONTEXT_BLOB ** context,
  * @param[in,out] context The context of the digest object.
  */
 void
-iesys_cryptossl_hash_abort(IESYS_CRYPTO_CONTEXT_BLOB ** context)
+iesys_cryptossl_hash_abort(
+    ESYS_CRYPTO_CONTEXT_BLOB **context,
+    void *userdata)
 {
+    UNUSED(userdata);
+
     LOG_TRACE("called for context-pointer %p", context);
     if (context == NULL || *context == NULL) {
         LOG_DEBUG("Null-Pointer passed");
@@ -366,10 +358,13 @@ iesys_cryptossl_hash_abort(IESYS_CRYPTO_CONTEXT_BLOB ** context)
  * @retval TSS2_ESYS_RC_GENERAL_FAILURE for errors of the crypto library.
  */
 TSS2_RC
-iesys_cryptossl_hmac_start(IESYS_CRYPTO_CONTEXT_BLOB ** context,
+iesys_cryptossl_hmac_start(ESYS_CRYPTO_CONTEXT_BLOB ** context,
                            TPM2_ALG_ID hashAlg,
-                           const uint8_t * key, size_t size)
+                           const uint8_t * key, size_t size,
+                           void *userdata)
 {
+    UNUSED(userdata);
+
     TSS2_RC r = TSS2_RC_SUCCESS;
     EVP_PKEY *hkey = NULL;
 
@@ -415,7 +410,7 @@ iesys_cryptossl_hmac_start(IESYS_CRYPTO_CONTEXT_BLOB ** context,
 
     mycontext->type = IESYS_CRYPTOSSL_TYPE_HMAC;
 
-    *context = (IESYS_CRYPTO_CONTEXT_BLOB *) mycontext;
+    *context = (ESYS_CRYPTO_CONTEXT_BLOB *) mycontext;
 
     EVP_PKEY_free(hkey);
 
@@ -439,9 +434,12 @@ iesys_cryptossl_hmac_start(IESYS_CRYPTO_CONTEXT_BLOB ** context,
  * @retval TSS2_ESYS_RC_BAD_REFERENCE for invalid parameters.
  */
 TSS2_RC
-iesys_cryptossl_hmac_update(IESYS_CRYPTO_CONTEXT_BLOB * context,
-                            const uint8_t * buffer, size_t size)
+iesys_cryptossl_hmac_update(ESYS_CRYPTO_CONTEXT_BLOB * context,
+                            const uint8_t * buffer, size_t size,
+                            void *userdata)
 {
+    UNUSED(userdata);
+
     LOG_TRACE("called for context %p, buffer %p and size %zd",
               context, buffer, size);
     if (context == NULL || buffer == NULL) {
@@ -462,26 +460,6 @@ iesys_cryptossl_hmac_update(IESYS_CRYPTO_CONTEXT_BLOB * context,
     return TSS2_RC_SUCCESS;
 }
 
-/** Update and HMAC digest value from a TPM2B object.
- *
- * The context of a digest object will be updated according to the hash
- * algorithm and the key of the context.
- * @param[in,out] context The context of the digest object which will be updated.
- * @param[in] b The TPM2B object for the update.
- * @retval TSS2_RC_SUCCESS on success.
- * @retval TSS2_ESYS_RC_BAD_REFERENCE for invalid parameters.
- */
-TSS2_RC
-iesys_cryptossl_hmac_update2b(IESYS_CRYPTO_CONTEXT_BLOB * context, TPM2B * b)
-{
-    LOG_TRACE("called for context-pointer %p and 2b-pointer %p", context, b);
-    if (context == NULL || b == NULL) {
-        return_error(TSS2_ESYS_RC_BAD_REFERENCE, "Null-Pointer passed");
-    }
-    TSS2_RC ret = iesys_cryptossl_hmac_update(context, &b->buffer[0], b->size);
-    return ret;
-}
-
 /** Write the HMAC digest value to a byte buffer and close the context.
  *
  * The digest value will written to a passed buffer and the resources of the
@@ -495,9 +473,11 @@ iesys_cryptossl_hmac_update2b(IESYS_CRYPTO_CONTEXT_BLOB * context, TPM2B * b)
  * @retval TSS2_ESYS_RC_GENERAL_FAILURE for errors of the crypto library.
  */
 TSS2_RC
-iesys_cryptossl_hmac_finish(IESYS_CRYPTO_CONTEXT_BLOB ** context,
-                            uint8_t * buffer, size_t * size)
+iesys_cryptossl_hmac_finish(ESYS_CRYPTO_CONTEXT_BLOB ** context,
+                            uint8_t * buffer, size_t * size,
+                            void *userdata)
 {
+    UNUSED(userdata);
 
     TSS2_RC r = TSS2_RC_SUCCESS;
 
@@ -528,38 +508,18 @@ iesys_cryptossl_hmac_finish(IESYS_CRYPTO_CONTEXT_BLOB ** context,
     return r;
 }
 
-/** Write the HMAC digest value to a TPM2B object and close the context.
- *
- * The digest value will written to a passed TPM2B object and the resources of
- * the HMAC object are released.
- * @param[in,out] context The context of the HMAC object.
- * @param[out] hmac The buffer for the digest value (caller-allocated).
- * @retval TSS2_RC_SUCCESS on success.
- * @retval TSS2_ESYS_RC_BAD_REFERENCE for invalid parameters.
- * @retval TSS2_ESYS_RC_BAD_SIZE if the size passed is lower than the HMAC length.
- * @retval TSS2_ESYS_RC_GENERAL_FAILURE for errors of the crypto library.
- */
-TSS2_RC
-iesys_cryptossl_hmac_finish2b(IESYS_CRYPTO_CONTEXT_BLOB ** context, TPM2B * hmac)
-{
-    LOG_TRACE("called for context-pointer %p and 2b-pointer %p", context, hmac);
-    if (context == NULL || *context == NULL || hmac == NULL) {
-        return_error(TSS2_ESYS_RC_BAD_REFERENCE, "Null-Pointer passed");
-    }
-    size_t s = hmac->size;
-    TSS2_RC ret = iesys_cryptossl_hmac_finish(context, &hmac->buffer[0], &s);
-    hmac->size = s;
-    return ret;
-}
-
 /** Release the resources of an HAMC object.
  *
  * The assigned resources will be released and the context will be set to NULL.
  * @param[in,out] context The context of the HMAC object.
  */
 void
-iesys_cryptossl_hmac_abort(IESYS_CRYPTO_CONTEXT_BLOB ** context)
+iesys_cryptossl_hmac_abort(
+    ESYS_CRYPTO_CONTEXT_BLOB **context,
+    void *userdata)
 {
+    UNUSED(userdata);
+
     LOG_TRACE("called for context-pointer %p", context);
     if (context == NULL || *context == NULL) {
         LOG_DEBUG("Null-Pointer passed");
@@ -588,8 +548,13 @@ iesys_cryptossl_hmac_abort(IESYS_CRYPTO_CONTEXT_BLOB ** context)
  * NOTE: the TPM should not be used to obtain the random data
  */
 TSS2_RC
-iesys_cryptossl_random2b(TPM2B_NONCE * nonce, size_t num_bytes)
+iesys_cryptossl_random2b(
+    TPM2B_NONCE *nonce,
+    size_t num_bytes,
+    void *userdata)
 {
+    UNUSED(userdata);
+
     int rc;
 #if OPENSSL_VERSION_NUMBER < 0x30000000L
     const RAND_METHOD *rand_save = RAND_get_rand_method();
@@ -601,7 +566,7 @@ iesys_cryptossl_random2b(TPM2B_NONCE * nonce, size_t num_bytes)
 #endif
 
     if (num_bytes == 0) {
-        nonce->size = sizeof(TPMU_HA);
+        nonce->size = sizeof(nonce->buffer);
     } else {
         nonce->size = num_bytes;
     }
@@ -640,8 +605,12 @@ iesys_cryptossl_pk_encrypt(TPM2B_PUBLIC * pub_tpm_key,
                            BYTE * in_buffer,
                            size_t max_out_size,
                            BYTE * out_buffer,
-                           size_t * out_size, const char *label)
+                           size_t * out_size,
+                           const char *label,
+                           void *userdata)
 {
+    UNUSED(userdata);
+
 #if OPENSSL_VERSION_NUMBER < 0x30000000L
     RSA *rsa_key = NULL;
     const EVP_MD * hashAlg = NULL;
@@ -905,8 +874,11 @@ iesys_cryptossl_get_ecdh_point(TPM2B_PUBLIC *key,
                                TPM2B_ECC_PARAMETER *Z,
                                TPMS_ECC_POINT *Q,
                                BYTE * out_buffer,
-                               size_t * out_size)
+                               size_t * out_size,
+                               void *userdata)
 {
+    UNUSED(userdata);
+
     TSS2_RC r = TSS2_RC_SUCCESS;
     EC_GROUP *group = NULL;               /* Group defines the used curve */
     EVP_PKEY_CTX *ctx = NULL;
@@ -1097,8 +1069,11 @@ iesys_cryptossl_sym_aes_encrypt(uint8_t * key,
                                 TPM2_ALG_ID tpm_mode,
                                 uint8_t * buffer,
                                 size_t buffer_size,
-                                uint8_t * iv)
+                                uint8_t * iv,
+                                void *userdata)
 {
+    UNUSED(userdata);
+
     TSS2_RC r = TSS2_RC_SUCCESS;
     const EVP_CIPHER  *cipher_alg = NULL;
     EVP_CIPHER_CTX *ctx = NULL;
@@ -1177,8 +1152,11 @@ iesys_cryptossl_sym_aes_decrypt(uint8_t * key,
                                 TPM2_ALG_ID tpm_mode,
                                 uint8_t * buffer,
                                 size_t buffer_size,
-                                uint8_t * iv)
+                                uint8_t * iv,
+                                void *userdata)
 {
+    UNUSED(userdata);
+
     TSS2_RC r = TSS2_RC_SUCCESS;
     const EVP_CIPHER *cipher_alg = NULL;
     EVP_CIPHER_CTX *ctx = NULL;
@@ -1244,6 +1222,9 @@ iesys_cryptossl_sym_aes_decrypt(uint8_t * key,
  * a return code.
  */
 TSS2_RC
-iesys_cryptossl_init() {
+iesys_cryptossl_init(void *userdata)
+{
+    UNUSED(userdata);
+
     return TSS2_RC_SUCCESS;
 }
