@@ -20,7 +20,7 @@
  *
  * Serialize the metadata of an ESYS_TR object into a byte buffer such that it
  * can be stored on disk for later use by a different program or context.
- * The serialized object can be deserialized suing Esys_TR_Deserialize.
+ * The serialized object can be deserialized using Esys_TR_Deserialize.
  * @param esys_context [in,out] The ESYS_CONTEXT.
  * @param esys_handle [in] The ESYS_TR object to serialize.
  * @param buffer [out] The buffer containing the serialized metadata.
@@ -483,7 +483,8 @@ Esys_TR_SetAuth(ESYS_CONTEXT * esys_context, ESYS_TR esys_handle,
         esys_object->auth = *authValue;
         /* Adapt auth value to hash for large auth values. */
         if (name_alg != TPM2_ALG_NULL) {
-            r = iesys_hash_long_auth_values(&esys_object->auth, name_alg);
+            r = iesys_hash_long_auth_values(&esys_context->crypto_backend,
+                    &esys_object->auth, name_alg);
             return_if_error(r, "Hashing overlength authValue failed.");
         }
     }
@@ -525,12 +526,14 @@ Esys_TR_GetName(ESYS_CONTEXT * esys_context, ESYS_TR esys_handle,
         return TSS2_ESYS_RC_MEMORY;
     }
     if (esys_object->rsrc.rsrcType == IESYSC_KEY_RSRC) {
-        r = iesys_get_name(&esys_object->rsrc.misc.rsrc_key_pub, *name);
+        r = iesys_get_name(&esys_context->crypto_backend,
+                &esys_object->rsrc.misc.rsrc_key_pub, *name);
         goto_if_error(r, "Error get name", error_cleanup);
 
     } else {
         if (esys_object->rsrc.rsrcType == IESYSC_NV_RSRC) {
-            r = iesys_nv_get_name(&esys_object->rsrc.misc.rsrc_nv_pub, *name);
+            r = iesys_nv_get_name(&esys_context->crypto_backend,
+                    &esys_object->rsrc.misc.rsrc_nv_pub, *name);
             goto_if_error(r, "Error get name", error_cleanup);
 
         } else {
