@@ -21,15 +21,21 @@
 #include "fapi_int.h"
 #include "fapi_crypto.h"
 #include "fapi_policy.h"
+#include "ifapi_curl.h"
 #include "ifapi_get_intl_cert.h"
 #include "ifapi_helpers.h"
+#include "tss2_mu.h"
 
 #define LOGMODULE fapi
 #include "util/log.h"
 #include "util/aux_util.h"
 
 #define EK_CERT_RANGE (0x01c07fff)
-
+#define RSA_EK_NONCE_NV_INDEX 0x01c00003
+#define RSA_EK_TEMPLATE_NV_INDEX 0x01c00004
+#define ECC_EK_NONCE_NV_INDEX 0x01c0000b
+#define ECC_EK_TEMPLATE_NV_INDEX 0x01c0000c
+#define ECC_SM2_EK_TEMPLATE_NV_INDEX 0x01c0001b
 
 /** Error cleanup of provisioning
  *
@@ -357,7 +363,10 @@ Fapi_Provision_Finish(FAPI_CONTEXT *context)
     TPM2B_NAME *srk_name = NULL, *srk_name_persistent = NULL;
     ESYS_TR srk_persistent_handle;
     ESYS_TR ek_handle = ESYS_TR_NONE;
-
+    UINT8* ek_template = NULL;
+    size_t ek_template_size;
+    UINT8* ek_nonce = NULL;
+    size_t ek_nonce_size;
 
     switch (context->state) {
         /* Read all hierarchies from keystore. */
@@ -399,32 +408,32 @@ Fapi_Provision_Finish(FAPI_CONTEXT *context)
             /* Use the first appropriate hierarchy for provisioning. The first found
                hierarchy will be copied into the provisioning context.*/
             if (strcmp(path, "/HS") == 0) {
-                command->hierarchies[command->path_idx].handle = ESYS_TR_RH_OWNER;
-                if (!hierarchy_hs->handle) {
+                command->hierarchies[command->path_idx].public.handle = ESYS_TR_RH_OWNER;
+                if (!hierarchy_hs->public.handle) {
                     r = ifapi_copy_ifapi_hierarchy_object(hierarchy_hs,
                                                           &command->
                                                           hierarchies[command->path_idx]);
                     goto_if_error(r, "Copy hierarchy", error_cleanup);
                 }
             } else if (strcmp(path, "/HE") == 0) {
-                command->hierarchies[command->path_idx].handle = ESYS_TR_RH_ENDORSEMENT;
-                if (!hierarchy_he->handle) {
+                command->hierarchies[command->path_idx].public.handle = ESYS_TR_RH_ENDORSEMENT;
+                if (!hierarchy_he->public.handle) {
                     r = ifapi_copy_ifapi_hierarchy_object(hierarchy_he,
                                                           &command->
                                                           hierarchies[command->path_idx]);
                     goto_if_error(r, "Copy hierarchy", error_cleanup);
                 }
             } else if (strcmp(path, "/HN") == 0) {
-                command->hierarchies[command->path_idx].handle = ESYS_TR_RH_NULL;
-                if (!hierarchy_hn->handle) {
+                command->hierarchies[command->path_idx].public.handle = ESYS_TR_RH_NULL;
+                if (!hierarchy_hn->public.handle) {
                     r = ifapi_copy_ifapi_hierarchy_object(hierarchy_hn,
                                                           &command->
                                                           hierarchies[command->path_idx]);
                     goto_if_error(r, "Copy hierarchy", error_cleanup);
                 }
             } else if (strcmp(path, "/LOCKOUT") == 0) {
-                command->hierarchies[command->path_idx].handle = ESYS_TR_RH_LOCKOUT;
-                if (!hierarchy_lockout->handle) {
+                command->hierarchies[command->path_idx].public.handle = ESYS_TR_RH_LOCKOUT;
+                if (!hierarchy_lockout->public.handle) {
                     r = ifapi_copy_ifapi_hierarchy_object(hierarchy_lockout,
                                                           &command->
                                                           hierarchies[command->path_idx]);
@@ -566,13 +575,75 @@ Fapi_Provision_Finish(FAPI_CONTEXT *context)
                     &command->public_templ);
             goto_if_error(r, "Merging profile", error_cleanup);
 
+            command->template_nv_index = 0;
+
+            if (context->profiles.default_profile.ignore_ek_template == TPM2_NO) {
+                if (context->profiles.default_profile.type == TPM2_ALG_RSA &&
+                    context->profiles.default_profile.keyBits == 2048) {
+                    command->template_nv_index = RSA_EK_TEMPLATE_NV_INDEX;
+                } else if (context->profiles.default_profile.type == TPM2_ALG_ECC) {
+                    if (context->profiles.default_profile.curveID == TPM2_ECC_NIST_P256) {
+                        command->template_nv_index = ECC_EK_TEMPLATE_NV_INDEX;
+                    } else if  (context->profiles.default_profile.curveID == TPM2_ECC_SM2_P256) {
+                        command->template_nv_index = ECC_SM2_EK_TEMPLATE_NV_INDEX;
+                    }
+                }
+            }
+
+            nvCmd->nv_read_state = NV_READ_CHECK_HANDLE;
+            nvCmd->tpm_handle = command->template_nv_index;
+            fallthrough;
+
+        statecase(context->state, PROVISION_READ_EK_TEMPLATE);
+            if (command->template_nv_index) {
+                /* Overwrite template with template from nv if available. */
+                r = ifapi_nv_read(context, &ek_template, &ek_template_size);
+                return_try_again(r);
+                goto_if_error_reset_state(r, "Read EK template", error_cleanup);
+
+                if (ek_template_size) {
+                    r = Tss2_MU_TPMT_PUBLIC_Unmarshal(ek_template, ek_template_size,
+                                           NULL, &command->public_templ.public.publicArea);
+                    SAFE_FREE(ek_template);
+                    goto_if_error_reset_state(r, "Unmarshal EK template", error_cleanup);
+                }
+            }
+            command->nonce_nv_index = 0;
+
+            if (context->profiles.default_profile.type == TPM2_ALG_RSA) {
+                    command->nonce_nv_index = RSA_EK_NONCE_NV_INDEX;
+            } else if  (context->profiles.default_profile.type == TPM2_ALG_ECC &&
+                        context->profiles.default_profile.curveID == TPM2_ECC_NIST_P256){
+                command->nonce_nv_index = ECC_EK_NONCE_NV_INDEX;
+            }
+
+            nvCmd->nv_read_state = NV_READ_CHECK_HANDLE;
+            nvCmd->tpm_handle = command->nonce_nv_index;
+
+            fallthrough;
+
+        statecase(context->state, PROVISION_READ_EK_NONCE);
+            ek_nonce_size = 0;
+            if (command->nonce_nv_index) {
+                r = ifapi_nv_read(context, &ek_nonce, &ek_nonce_size);
+                return_try_again(r);
+                goto_if_error_reset_state(r, "Read EK nonce", error_cleanup);
+            }
+
             /* Clear key object for the primary to be created */
             memset(pkey, 0, sizeof(IFAPI_KEY));
 
+            if (ek_nonce_size) {
+                pkey->nonce.size = ek_nonce_size;
+                memcpy(&pkey->nonce.buffer[0], ek_nonce, ek_nonce_size);
+                SAFE_FREE(ek_nonce);
+            }
+
             /* Prepare the EK generation. */
             r = ifapi_init_primary_async(context, TSS2_EK);
             goto_if_error(r, "Initialize primary", error_cleanup);
             fallthrough;
+
         statecase(context->state, PROVISION_AUTH_EK_AUTH_SENT);
             fallthrough;
 
@@ -583,7 +654,6 @@ Fapi_Provision_Finish(FAPI_CONTEXT *context)
 
             /* Check whether a persistent EK handle was defined in profile. */
             if (command->public_templ.persistent_handle) {
-
                 pkey->persistent_handle = command->public_templ.persistent_handle;
 
                 if (hierarchy_hs->misc.hierarchy.with_auth == TPM2_YES ||
@@ -604,8 +674,8 @@ Fapi_Provision_Finish(FAPI_CONTEXT *context)
             fallthrough;
 
         statecase(context->state, PROVISION_PREPARE_EK_EVICT);
-            r = Esys_EvictControl_Async(context->esys, hierarchy_hs->handle,
-                     pkeyObject->handle, ESYS_TR_PASSWORD, ESYS_TR_NONE,
+            r = Esys_EvictControl_Async(context->esys, hierarchy_hs->public.handle,
+                     pkeyObject->public.handle, ESYS_TR_PASSWORD, ESYS_TR_NONE,
                      ESYS_TR_NONE, pkey->persistent_handle);
 
             goto_if_error(r, "Error Esys EvictControl", error_cleanup);
@@ -839,9 +909,35 @@ Fapi_Provision_Finish(FAPI_CONTEXT *context)
 
             fallthrough;
 
+        statecase(context->state, PROVISION_PREPARE_READ_INT_CERT);
+	    const char *int_ca_file;
+
+            /* Prepare reading of intermediate certificate. */
+            int_ca_file = NULL;
+#ifdef SELF_GENERATED_CERTIFICATE
+#pragma message ( "*** Allow self generated certifcate ***" )
+            int_ca_file = getenv("FAPI_TEST_INT_CERT");
+#endif
+            if (!int_ca_file) {
+                context->state = PROVISION_EK_CHECK_CERT;
+                return TSS2_FAPI_RC_TRY_AGAIN;
+            }
+            r = ifapi_io_read_async(&context->io, int_ca_file);
+            return_try_again(r);
+            goto_if_error2(r, "Reading certificate %s", error_cleanup, int_ca_file);
+
+	        fallthrough;
+
+        statecase(context->state, PROVISION_READ_INT_CERT);
+            r = ifapi_io_read_finish(&context->io, (uint8_t **) &command->intermed_crt, NULL);
+            return_try_again(r);
+            goto_if_error(r, "Reading intermediate certificate failed", error_cleanup);
+
+            fallthrough;
+
         statecase(context->state, PROVISION_EK_CHECK_CERT);
             /* The EK certificate will be verified against the FAPI list of root certificates. */
-            r = ifapi_verify_ek_cert(command->root_crt, command->intermed_crt, command->pem_cert);
+            r = ifapi_curl_verify_ek_cert(command->root_crt, command->intermed_crt, command->pem_cert);
             SAFE_FREE(command->root_crt);
             SAFE_FREE(command->intermed_crt);
             goto_if_error2(r, "Verify EK certificate", error_cleanup);
@@ -1030,8 +1126,8 @@ Fapi_Provision_Finish(FAPI_CONTEXT *context)
                 pkey->persistent_handle = command->public_templ.persistent_handle;
 
                 /* Prepare making the SRK permanent. */
-                r = Esys_EvictControl_Async(context->esys, hierarchy_hs->handle,
-                    pkeyObject->handle, ESYS_TR_PASSWORD, ESYS_TR_NONE, ESYS_TR_NONE,
+                r = Esys_EvictControl_Async(context->esys, hierarchy_hs->public.handle,
+                    pkeyObject->public.handle, ESYS_TR_PASSWORD, ESYS_TR_NONE, ESYS_TR_NONE,
                     pkey->persistent_handle);
                 goto_if_error(r, "Error Esys EvictControl", error_cleanup);
 
@@ -1127,33 +1223,34 @@ Fapi_Provision_Finish(FAPI_CONTEXT *context)
             return TSS2_FAPI_RC_TRY_AGAIN;
 
         statecase(context->state, PROVISION_WAIT_FOR_SRK_PERSISTENT);
-            r = Esys_EvictControl_Finish(context->esys, &pkeyObject->handle);
+            r = Esys_EvictControl_Finish(context->esys, &pkeyObject->public.handle);
             return_try_again(r);
             goto_if_error(r, "Evict control failed", error_cleanup);
 
             /* The SRK was made persistent and can be written to key store. */
             command->srk_tpm_handle = pkeyObject->misc.key.persistent_handle;
-            command->srk_esys_handle = pkeyObject->handle;
+            command->srk_esys_handle = pkeyObject->public.handle;
             context->state = PROVISION_SRK_WRITE_PREPARE;
             return TSS2_FAPI_RC_TRY_AGAIN;
 
         statecase(context->state, PROVISION_WAIT_FOR_EK_PERSISTENT);
-            ek_handle = pkeyObject->handle;
-            r = Esys_EvictControl_Finish(context->esys, &pkeyObject->handle);
+            ek_handle = pkeyObject->public.handle;
+            r = Esys_EvictControl_Finish(context->esys, &pkeyObject->public.handle);
             return_try_again(r);
 
             if (number_rc(r) == TPM2_RC_BAD_AUTH
                 && hierarchy_hs->misc.hierarchy.with_auth == TPM2_NO) {
                 hierarchy_hs->misc.hierarchy.with_auth = TPM2_YES;
                 /* Public handle was changed to 0xfff in the error case. */
-                pkeyObject->handle = ek_handle;
+                pkeyObject->public.handle = ek_handle;
                 context->state = PROVISION_AUTHORIZE_HS_FOR_EK_EVICT;
                 return TSS2_FAPI_RC_TRY_AGAIN;
             }
+
             goto_if_error(r, "Evict control failed", error_cleanup);
 
             command->ek_tpm_handle = pkeyObject->misc.key.persistent_handle;
-            command->ek_esys_handle = pkeyObject->handle;
+            command->ek_esys_handle = pkeyObject->public.handle;
 
             context->state = PROVISION_INIT_GET_CAP2;
             return TSS2_FAPI_RC_TRY_AGAIN;
@@ -1417,17 +1514,17 @@ Fapi_Provision_Finish(FAPI_CONTEXT *context)
                exist in the keystore. If all files are written the provisioning
                is continued at the appropriate next state. */
             if (command->path_idx == command->numHierarchyObjects) {
-                if (command->hierarchy->handle == ESYS_TR_RH_OWNER)
+                if (command->hierarchy->public.handle == ESYS_TR_RH_OWNER)
                     context->state = PROVISION_PREPARE_NULL;
-                if (command->hierarchy->handle == ESYS_TR_RH_NULL)
+                if (command->hierarchy->public.handle == ESYS_TR_RH_NULL)
                     context->state = PROVISION_FINISHED;
-                else if (command->hierarchy->handle == ESYS_TR_RH_ENDORSEMENT)
+                else if (command->hierarchy->public.handle == ESYS_TR_RH_ENDORSEMENT)
                     context->state = PROVISION_CHANGE_SH_CHECK;
-                else if (command->hierarchy->handle == ESYS_TR_RH_LOCKOUT)
+                else if (command->hierarchy->public.handle == ESYS_TR_RH_LOCKOUT)
                     context->state = PROVISION_CHANGE_EH_CHECK;
                 return TSS2_FAPI_RC_TRY_AGAIN;
             }
-            if (command->hierarchies[command->path_idx].handle == command->hierarchy->handle) {
+            if (command->hierarchies[command->path_idx].public.handle == command->hierarchy->public.handle) {
                 r = ifapi_keystore_store_async(&context->keystore, &context->io,
                                                command->pathlist[command->path_idx],
                                                command->hierarchy);
