@@ -21,10 +21,8 @@
 #include <openssl/param_build.h>
 #endif
 #include <openssl/x509v3.h>
-#include <curl/curl.h>
 #include <openssl/err.h>
 
-#include "fapi_certificates.h"
 #include "fapi_util.h"
 #include "util/aux_util.h"
 #include "fapi_crypto.h"
@@ -757,6 +755,41 @@ ifapi_ecc_der_sig_to_tpm(
     return TSS2_RC_SUCCESS;
 }
 
+/** Converts a HMAC byte buffer into the
+ * TPM format. It can then be verified by the TPM.
+ *
+ * @param[in]  byte buffer holding the HMAC
+ * @param[in]  signatureSize The size of signature in bytes
+ * @param[in]  Size The size of the verification key
+ * @param[in]  hashAlgorithm The TSS identifier of the hash algorithm to use
+ * @param[out] tpmSignature The signature in the TPM format
+ *
+ * @retval TSS2_RC_SUCCESS on success
+ * @retval TSS2_FAPI_RC_GENERAL_FAILURE if an error occurs in the crypto library
+ * @retval TSS2_FAPI_RC_BAD_REFERENCE if signature or tpmSignature is NULL
+ * @retval TSS2_FAPI_RC_MEMORY if memory could not be allocated
+ */
+static TSS2_RC
+ifapi_hmac_sig_to_tpm(
+    const unsigned char *signature,
+    size_t signatureSize,
+    TPMI_ALG_HASH hashAlgorithm,
+    TPMT_SIGNATURE *tpmSignature)
+{
+    /* Check for NULL parameters */
+    return_if_null(signature, "signature is NULL", TSS2_FAPI_RC_BAD_REFERENCE);
+    return_if_null(tpmSignature, "tpmSignature is NULL", TSS2_FAPI_RC_BAD_REFERENCE);
+
+    if (ifapi_hash_get_digest_size(hashAlgorithm) != signatureSize) {
+        return_error(TSS2_FAPI_RC_BAD_VALUE, "Invalid HMAC size");
+    }
+
+    tpmSignature->sigAlg = TPM2_ALG_HMAC;
+    tpmSignature->signature.hmac.hashAlg = hashAlgorithm;
+    memcpy(&tpmSignature->signature.hmac.digest.sha1, signature, signatureSize);
+    return TSS2_RC_SUCCESS;
+}
+
 /** Convert signature from DER to TPM format.
  *
  * The signature in DER format is converted to TPM format to
@@ -810,7 +843,12 @@ ifapi_der_sig_to_tpm(
         return ifapi_ecc_der_sig_to_tpm(signature, signatureSize,
                                         tpmPublic->unique.ecc.x.size, hashAlgorithm,
                                         tpmSignature);
-    } else {
+    } else if (tpmPublic->type == TPM2_ALG_KEYEDHASH) {
+         return ifapi_hmac_sig_to_tpm(signature, signatureSize,
+                                      hashAlgorithm,
+                                      tpmSignature);
+
+    }else {
         return_error(TSS2_FAPI_RC_BAD_VALUE, "Invalid key tpye.");
     }
     return TSS2_RC_SUCCESS;
@@ -1164,7 +1202,7 @@ get_ecc_tpm2b_public_from_evp(
 #endif
     default:
         goto_error(r, TSS2_FAPI_RC_BAD_VALUE,
-                   "Curve %x not implemented", cleanup, curveId);
+                   "Curve %i not implemented", cleanup, curveId);
     }
     tpmPublic->publicArea.parameters.eccDetail.curveID = tpmCurveId;
 
@@ -1730,67 +1768,6 @@ ifapi_crypto_hash_abort(IFAPI_CRYPTO_CONTEXT_BLOB **context)
     *context = NULL;
 }
 
-/**
- * Get url to download crl from certificate.
- * @retval TSS2_FAPI_RC_MEMORY if not enough memory can be allocated.
- * @retval TSS2_FAPI_RC_BAD_VALUE if an invalid value was passed into
- *         the function.
- * @retval TSS2_FAPI_RC_GENERAL_FAILURE if an internal error occurred.
- * @retval TSS2_FAPI_RC_NO_CERT if an error did occur during certificate downloading.
- */
-TSS2_RC
-get_crl_from_cert(X509 *cert, X509_CRL **crl)
-{
-    TSS2_RC r = TSS2_RC_SUCCESS;
-    unsigned char* url = NULL;
-    unsigned char *crl_buffer = NULL;
-    size_t crl_buffer_size;
-    int nid = NID_crl_distribution_points;
-    STACK_OF(DIST_POINT) * dist_points = (STACK_OF(DIST_POINT) *)X509_get_ext_d2i(cert, nid, NULL, NULL);
-    int curl_rc;
-
-    *crl = NULL;
-    for (int i = 0; i < sk_DIST_POINT_num(dist_points); i++)
-    {
-        DIST_POINT *dp = sk_DIST_POINT_value(dist_points, i);
-        DIST_POINT_NAME    *distpoint = dp->distpoint;
-        if (distpoint->type==0)
-        {
-            for (int j = 0; j < sk_GENERAL_NAME_num(distpoint->name.fullname); j++)
-            {
-                GENERAL_NAME *gen_name = sk_GENERAL_NAME_value(distpoint->name.fullname, j);
-                ASN1_IA5STRING *asn1_str = gen_name->d.uniformResourceIdentifier;
-                SAFE_FREE(url);
-                url = (unsigned char *)strdup((char *)asn1_str->data);
-                goto_if_null2(url, "Out of memory", r, TSS2_FAPI_RC_MEMORY, cleanup);
-            }
-        }
-    }
-
-    /* No CRL dist point in the cert is legitimate */
-    if (url == NULL) {
-        goto cleanup;
-    }
-
-    curl_rc = ifapi_get_curl_buffer(url, &crl_buffer, &crl_buffer_size);
-    if (curl_rc != 0) {
-        goto_error(r, TSS2_FAPI_RC_NO_CERT, "Get crl.", cleanup);
-    }
-
-    unsigned const char* tmp_ptr1 = crl_buffer;
-    unsigned const char** tmp_ptr2 = &tmp_ptr1;
-
-    if (!d2i_X509_CRL(crl, tmp_ptr2, crl_buffer_size)) {
-        goto_error(r, TSS2_FAPI_RC_BAD_VALUE, "Can't convert crl.", cleanup);
-    }
-
-cleanup:
-    SAFE_FREE(crl_buffer);
-    CRL_DIST_POINTS_free(dist_points);
-    SAFE_FREE(url);
-    return r;
-}
-
 /**
  * Converts a TPM certificate buffer to the PEM format.
  *
@@ -1993,218 +1970,6 @@ cleanup:
     return r;
 }
 
-/** Convert buffer from DER format to X509 certificate.
- *
- * @param[in] cert_buffer Certificate in DER format.
- * @aparm[in] cert_buffer_size Size of DER certificate.
- * @retval X509 OSSL certificate object.
- * @retval NULL If the conversion fails.
- */
-static X509
-*get_cert_from_buffer(unsigned char *cert_buffer, size_t cert_buffer_size)
-{
-    unsigned char *buffer = cert_buffer;
-    X509 *cert = NULL;
-
-    unsigned const char* tmp_ptr1 = buffer;
-    unsigned const char** tmp_ptr2 = &tmp_ptr1;
-
-    if (!d2i_X509(&cert, tmp_ptr2, cert_buffer_size))
-        return NULL;
-    return cert;
-}
-
-/**
- * Verify EK certificate read from TPM.
- *
- * @param[in] root_cert_pem The vendor root certificate.
- * @param[in] intermed_cert_pem The vendor intermediate certificate.
- * @param[in] ek_cert_pem The ek certificate from TPM.
- *
- * @retval TSS2_RC_SUCCESS on success
- * @retval TSS2_FAPI_RC_BAD_VALUE if the verification was no successful.
- * @retval TSS2_FAPI_RC_NO_CERT if an error did occur during certificate downloading.
- * @retval TSS2_FAPI_RC_GENERAL_FAILURE if an internal error occurred.
- * @retval TSS2_FAPI_RC_MEMORY if not enough memory can be allocated.
- */
-TSS2_RC
-ifapi_verify_ek_cert(
-    char* root_cert_pem,
-    char* intermed_cert_pem,
-    char* ek_cert_pem)
-{
-    TSS2_RC r = TSS2_RC_SUCCESS;
-    X509 *root_cert = NULL;
-    X509 *intermed_cert = NULL;
-    X509 *ek_cert = NULL;
-    X509_STORE *store = NULL;
-    X509_STORE_CTX *ctx = NULL;
-    X509_CRL *crl_intermed = NULL;
-    X509_CRL *crl_ek = NULL;
-    int i;
-    size_t ui;
-    AUTHORITY_INFO_ACCESS *info = NULL;
-    ASN1_IA5STRING *uri = NULL;
-    unsigned char * url;
-    unsigned char *cert_buffer = NULL;
-    size_t cert_buffer_size;
-    int curl_rc;
-
-    LOG_DEBUG("EK Certificate: %s", ek_cert_pem);
-
-    ek_cert = get_X509_from_pem(ek_cert_pem);
-    goto_if_null2(ek_cert, "Failed to convert PEM certificate to DER.",
-                  r, TSS2_FAPI_RC_BAD_VALUE, cleanup);
-
-    if (intermed_cert_pem) {
-        intermed_cert = get_X509_from_pem(intermed_cert_pem);
-        goto_if_null2(intermed_cert, "Failed to convert PEM certificate to DER.",
-                      r, TSS2_FAPI_RC_BAD_VALUE, cleanup);
-    } else {
-        /* Get uri for ek intermediate certificate. */
-        info = X509_get_ext_d2i(ek_cert, NID_info_access, NULL, NULL);
-
-        for (i = 0; i < sk_ACCESS_DESCRIPTION_num(info); i++) {
-            ACCESS_DESCRIPTION *ad = sk_ACCESS_DESCRIPTION_value(info, i);
-            if (ad->location->type != GEN_URI) {
-                continue;
-            }
-            uri = ad->location->d.uniformResourceIdentifier;
-            url = uri->data;
-            curl_rc = ifapi_get_curl_buffer(url, &cert_buffer, &cert_buffer_size);
-            if (curl_rc != 0) {
-                goto_error(r, TSS2_FAPI_RC_NO_CERT, "Get certificate.", cleanup);
-            }
-            goto_if_null2(cert_buffer, "No certificate downloaded", r,
-                          TSS2_FAPI_RC_NO_CERT, cleanup);
-            LOGBLOB_DEBUG(cert_buffer, cert_buffer_size, "Intermediate certificate:");
-        }
-        goto_if_null2(cert_buffer, "No certificate downloaded", r,
-                      TSS2_FAPI_RC_NO_CERT, cleanup);
-
-        intermed_cert = get_cert_from_buffer(cert_buffer, cert_buffer_size);
-
-        SAFE_FREE(cert_buffer);
-        if (!intermed_cert) {
-            LOGBLOB_ERROR(cert_buffer, cert_buffer_size,
-                          "Failed to convert intermediate certificate to X509 format.");
-            r = TSS2_FAPI_RC_GENERAL_FAILURE;
-            goto cleanup;
-        }
-
-         /* Get Certificate revocation list for Intermediate certificate */
-        r = get_crl_from_cert(intermed_cert, &crl_intermed);
-        goto_if_error(r, "Get crl for intermediate certificate.", cleanup);
-
-        /* Get Certificate revocation list for EK certificate */
-        r = get_crl_from_cert(ek_cert, &crl_ek);
-        goto_if_error(r, "Get crl for ek certificate.", cleanup);
-    }
-
-    /* Prepare X509 certificate store */
-
-    store = X509_STORE_new();
-
-    goto_if_null2(store, "Failed to create X509 store.",
-                  r, TSS2_FAPI_RC_GENERAL_FAILURE, cleanup);
-
-    /* Add Certificate revocation list for EK certificate if one exists. */
-    if (crl_ek) {
-        /* Set the flags of the store to use CRLs. */
-        X509_STORE_set_flags(store, X509_V_FLAG_CRL_CHECK | X509_V_FLAG_CRL_CHECK_ALL);
-        if (1 != X509_STORE_add_crl(store, crl_ek)) {
-            goto_error(r, TSS2_FAPI_RC_GENERAL_FAILURE,
-                       "Failed to add intermediate crl.", cleanup);
-        }
-    }
-
-    /* Add Certificate revocation list for intermediate certificate if one exists. */
-    if (crl_intermed) {
-        /* Set the flags of the store to use CRLs. */
-        X509_STORE_set_flags(store, X509_V_FLAG_CRL_CHECK | X509_V_FLAG_CRL_CHECK_ALL);
-        if (1 != X509_STORE_add_crl(store, crl_intermed)) {
-            goto_error(r, TSS2_FAPI_RC_GENERAL_FAILURE,
-                       "Failed to add intermediate crl.", cleanup);
-        }
-    }
-
-    /* Add stored root certificates */
-    for (ui = 0; ui < sizeof(root_cert_list) / sizeof(char *); ui++) {
-         root_cert = get_X509_from_pem(root_cert_list[ui]);
-         goto_if_null2(root_cert, "Failed to convert PEM certificate to DER.",
-                       r, TSS2_FAPI_RC_BAD_VALUE, cleanup);
-         if (1 != X509_STORE_add_cert(store, root_cert)) {
-             goto_error(r, TSS2_FAPI_RC_GENERAL_FAILURE,
-                        "Failed to add root certificate", cleanup);
-        }
-        OSSL_FREE(root_cert, X509);
-    }
-
-    /* Create root cert if passed as parameter */
-    if (root_cert_pem) {
-        root_cert = get_X509_from_pem(root_cert_pem);
-        goto_if_null2(root_cert, "Failed to convert PEM certificate to DER.",
-                      r, TSS2_FAPI_RC_BAD_VALUE, cleanup);
-
-        if (1 != X509_STORE_add_cert(store, root_cert)) {
-            goto_error(r, TSS2_FAPI_RC_GENERAL_FAILURE,
-                       "Failed to add root certificate", cleanup);
-        }
-        OSSL_FREE(root_cert, X509);
-    }
-
-    /* Verify intermediate certificate */
-    ctx = X509_STORE_CTX_new();
-    goto_if_null2(ctx, "Failed to create X509 store context.",
-                  r, TSS2_FAPI_RC_GENERAL_FAILURE, cleanup);
-    if (1 != X509_STORE_CTX_init(ctx, store, intermed_cert, NULL)) {
-        goto_error(r, TSS2_FAPI_RC_GENERAL_FAILURE,
-                   "Failed to initialize X509 context.", cleanup);
-    }
-    if (1 != X509_verify_cert(ctx)) {
-        LOG_ERROR("%s", X509_verify_cert_error_string(X509_STORE_CTX_get_error(ctx)));
-        goto_error(r, TSS2_FAPI_RC_GENERAL_FAILURE,
-                   "Failed to verify intermediate certificate", cleanup);
-    }
-    if (1 != X509_STORE_add_cert(store, intermed_cert)) {
-        goto_error(r, TSS2_FAPI_RC_GENERAL_FAILURE,
-                   "Failed to add intermediate certificate", cleanup);
-    }
-
-    X509_STORE_CTX_cleanup(ctx);
-    X509_STORE_CTX_free(ctx);
-    ctx = NULL;
-    ctx = X509_STORE_CTX_new();
-    goto_if_null2(ctx, "Failed to create X509 store context.",
-                  r, TSS2_FAPI_RC_GENERAL_FAILURE, cleanup);
-
-    if (1 != X509_STORE_CTX_init(ctx, store, ek_cert, NULL)) {
-        goto_error(r, TSS2_FAPI_RC_GENERAL_FAILURE,
-                   "Failed to initialize X509 context.", cleanup);
-    }
-    /* Verify the EK certificate. */
-    if (1 != X509_verify_cert(ctx)) {
-        LOG_ERROR("%s", X509_verify_cert_error_string(X509_STORE_CTX_get_error(ctx)));
-        goto_error(r, TSS2_FAPI_RC_GENERAL_FAILURE,
-                   "Failed to verify EK certificate", cleanup);
-    }
-
-cleanup:
-    if (ctx) {
-        X509_STORE_CTX_cleanup(ctx);
-        X509_STORE_CTX_free(ctx);
-    }
-    if (store)
-        X509_STORE_free(store);
-    OSSL_FREE(root_cert, X509);
-    OSSL_FREE(intermed_cert, X509);
-    OSSL_FREE(ek_cert, X509);
-    OSSL_FREE(crl_intermed, X509_CRL);
-    OSSL_FREE(crl_ek, X509_CRL);
-    OSSL_FREE(info, AUTHORITY_INFO_ACCESS);
-    return r;
-}
-
 /** Compute the fingerprint of a TPM public key.
  *
  * @param[in] tpmPublicKey The public key created by the TPM
@@ -2277,3 +2042,61 @@ cleanup:
     }
     return r;
 }
+
+
+/** Compute base64 string from binary data.
+ *
+ * @param[in] buffer The binary data.
+ * @param[in] buffer_size The size of the binary data.
+ * @param[out] b64_data The base64 encoded string.
+ *
+ * @retval TSS2_RC_SUCCESS on success
+ * @retval TSS2_FAPI_RC_GENERAL_FAILURE if an error occurs in the crypto library
+ * @retval TSS2_FAPI_RC_MEMORY if memory could not be allocated
+ * @retval TSS2_FAPI_RC_BAD_REFERENCE a invalid null pointer is passed.
+ */
+TSS2_RC
+ifapi_base64encode(uint8_t *buffer, size_t buffer_size, char** b64_data) {
+    BIO *bio64 = NULL;
+    BIO *bio = NULL;
+    TSS2_RC r = TPM2_RC_SUCCESS;
+    BUF_MEM *b64_mem;
+    int b64_size;
+    int bytes_written;
+
+    return_if_null(buffer, "Buffer to be encoded is NULL", TSS2_FAPI_RC_BAD_REFERENCE);
+    return_if_null(b64_data, "Pointer to store the result is NULL", TSS2_FAPI_RC_BAD_REFERENCE);
+
+    /* Memory IO will be used for OSSL key conversion */
+    bio64 = BIO_new(BIO_f_base64());
+    goto_if_null2(bio64, "Out of memory.", r, TSS2_FAPI_RC_MEMORY, cleanup);
+    bio = BIO_new(BIO_s_mem());
+    goto_if_null2(bio, "Out of memory.", r, TSS2_FAPI_RC_MEMORY, cleanup);
+    bio = BIO_push(bio64, bio);
+
+    BIO_set_flags(bio, BIO_FLAGS_BASE64_NO_NL);
+    bytes_written = BIO_write(bio, buffer, buffer_size);
+    if (bytes_written != (int)buffer_size) {
+        goto_error(r, TSS2_FAPI_RC_GENERAL_FAILURE, "Invalid BIO_write",
+                   cleanup);
+    }
+
+    BIO_flush(bio);
+    BIO_get_mem_ptr(bio, &b64_mem);
+    goto_if_null2(b64_mem, "Out of memory.", r, TSS2_FAPI_RC_MEMORY, cleanup);
+
+    b64_size = BIO_get_mem_data(bio, NULL);
+    *b64_data = malloc(b64_size + 1);
+    goto_if_null(*b64_data, "Out of memory.", TSS2_FAPI_RC_MEMORY,
+            cleanup);
+    memset(*b64_data, 0, b64_size + 1);
+    memcpy(*b64_data, (*b64_mem).data, b64_size);
+    BIO_free_all(bio);
+    return r;
+
+ cleanup:
+    if (bio)
+        BIO_free_all(bio);
+
+    return r;
+}
