@@ -478,9 +478,9 @@ ifapi_io_dirfiles(
     char ***files,
     size_t *numfiles)
 {
-    DIR *dir;
-    struct dirent *entry;
     char **paths;
+    int numentries = 0;
+    struct dirent **namelist;
     size_t numpaths = 0;
     check_not_null(dirname);
     check_not_null(files);
@@ -488,48 +488,44 @@ ifapi_io_dirfiles(
 
     LOG_TRACE("List directory: %s", dirname);
 
-    paths = calloc(10, sizeof(*paths));
-    check_oom(paths);
-
-    if (!(dir = opendir(dirname))) {
-        free(paths);
+    numentries = scandir(dirname, &namelist, NULL, alphasort);
+    if (numentries < 0) {
         return_error2(TSS2_FAPI_RC_IO_ERROR, "Could not open directory: %s",
                       dirname);
     }
 
+    paths = calloc(numentries, sizeof(*paths));
+    check_oom(paths);
+
     /* Iterating through the list of entries inside the directory. */
-    while ((entry = readdir(dir)) != NULL) {
-        LOG_TRACE("Looking at %s", entry->d_name);
-        if (entry->d_type != DT_REG)
+    for (size_t i = 0; i < (size_t) numentries; i++) {
+        LOG_TRACE("Looking at %s", namelist[i]->d_name);
+        if (namelist[i]->d_type != DT_REG)
             continue;
 
-        if (numpaths % 10 == 9) {
-#ifdef HAVE_REALLOCARRAY
-            paths = reallocarray(paths, numpaths + 10, sizeof(*paths));
-#else /* HAVE_REALLOCARRAY */
-            paths = realloc(paths, (numpaths + 10) * sizeof(*paths));
-#endif /* HAVE_REALLOCARRAY */
-            if (!paths)
-                closedir(dir);
-            check_oom(paths);
-        }
-
-        paths[numpaths] = strdup(entry->d_name);
+        paths[numpaths] = strdup(namelist[i]->d_name);
         if (!paths[numpaths])
             goto error_oom;
 
         LOG_TRACE("Added %s to the list at index %zi", paths[numpaths], numpaths);
         numpaths += 1;
     }
-    closedir(dir);
 
     *files = paths;
     *numfiles = numpaths;
 
+    for (int i = 0; i < numentries; i++) {
+        free(namelist[i]);
+    }
+    free(namelist);
+
     return TSS2_RC_SUCCESS;
 
 error_oom:
-    closedir(dir);
+    for (int i = 0; i < numentries; i++) {
+        free(namelist[i]);
+    }
+    free(namelist);
     LOG_ERROR("Out of memory");
     for (size_t i = 0; i < numpaths; i++)
         free(paths[i]);
