@@ -1659,7 +1659,7 @@ ifapi_copy_ifapi_key_object(IFAPI_OBJECT * dest, const IFAPI_OBJECT * src) {
 
     dest->objectType = src->objectType;
     dest->system = src->system;
-    dest->handle = src->handle;
+    dest->public.handle = src->public.handle;
     dest->authorization_state = src->authorization_state;
 
     return r;
@@ -1705,7 +1705,7 @@ ifapi_copy_ifapi_hierarchy_object(IFAPI_OBJECT * dest, const IFAPI_OBJECT * src)
 
     dest->objectType = src->objectType;
     dest->system = src->system;
-    dest->handle = src->handle;
+    dest->public.handle = src->public.handle;
     dest->authorization_state = src->authorization_state;
 
     return r;
