@@ -36,7 +36,7 @@
  */
 static void
 copy_policy_digest(TPML_DIGEST_VALUES *dest, TPML_DIGEST_VALUES *src,
-                   size_t digest_idx, size_t hash_size, char *txt)
+                   size_t digest_idx, size_t hash_size, char *txt MAYBE_UNUSED)
 {
     memcpy(&dest->digests[digest_idx].digest, &src->digests[digest_idx].digest,
            hash_size);
@@ -54,8 +54,10 @@ copy_policy_digest(TPML_DIGEST_VALUES *dest, TPML_DIGEST_VALUES *src,
  * @param[in] txt Text which will be used for additional logging information.
  */
 static void
-log_policy_digest(TPML_DIGEST_VALUES *dest, size_t digest_idx, size_t hash_size,
-                  char *txt)
+log_policy_digest(TPML_DIGEST_VALUES *dest MAYBE_UNUSED,
+                  size_t digest_idx MAYBE_UNUSED,
+                  size_t hash_size MAYBE_UNUSED,
+                  char *txt MAYBE_UNUSED)
 {
     LOGBLOB_DEBUG((uint8_t *)&dest->digests[digest_idx].digest, hash_size,
                   "Digest %s", txt);
@@ -287,7 +289,7 @@ ifapi_calculate_policy_authorize_nv(
 
     /* Written flag has to be set for policy calculation, because during
        policy execution it will be set. */
-    policy->nvPublic.nvPublic.attributes |= TPMA_NV_WRITTEN;
+    policy->nvPublic.attributes |= TPMA_NV_WRITTEN;
 
     r = ifapi_nv_get_name(&policy->nvPublic, &nv_name);
     return_if_error(r, "Compute NV name");
@@ -646,12 +648,10 @@ cleanup:
  */
 TSS2_RC
 ifapi_calculate_policy_physical_presence(
-    TPMS_POLICYPHYSICALPRESENCE *policy,
     TPML_DIGEST_VALUES *current_digest,
     TPMI_ALG_HASH current_hash_alg)
 {
     TSS2_RC r = TSS2_RC_SUCCESS;
-    (void)policy;
 
     LOG_DEBUG("call");
 
@@ -666,7 +666,6 @@ ifapi_calculate_policy_physical_presence(
  *
  * The policy will be updated with the function ifapi_calculate_simple_policy()
  *
- * @param[in] policy The policy auth value.
  * @param[in,out] current_digest The digest list which has to be updated.
  * @param[in] current_hash_alg The hash algorithm used for the policy computation.
  *
@@ -679,12 +678,10 @@ ifapi_calculate_policy_physical_presence(
  */
 TSS2_RC
 ifapi_calculate_policy_auth_value(
-    TPMS_POLICYAUTHVALUE *policy,
     TPML_DIGEST_VALUES *current_digest,
     TPMI_ALG_HASH current_hash_alg)
 {
     TSS2_RC r = TSS2_RC_SUCCESS;
-    (void)policy;
 
     LOG_DEBUG("call");
 
@@ -699,7 +696,6 @@ ifapi_calculate_policy_auth_value(
  *
  * The policy will be updated with the function ifapi_calculate_simple_policy()
  *
- * @param[in] policy The policy password.
  * @param[in,out] current_digest The digest list which has to be updated.
  * @param[in] current_hash_alg The hash algorithm used for the policy computation.
  *
@@ -712,12 +708,10 @@ ifapi_calculate_policy_auth_value(
  */
 TSS2_RC
 ifapi_calculate_policy_password(
-    TPMS_POLICYPASSWORD *policy,
     TPML_DIGEST_VALUES *current_digest,
     TPMI_ALG_HASH current_hash_alg)
 {
     TSS2_RC r = TSS2_RC_SUCCESS;
-    (void)policy;
 
     LOG_DEBUG("call");
 
@@ -1067,7 +1061,7 @@ ifapi_calculate_policy_nv(
 
     /* Written flag has to be set for policy calculation, because during
        policy execution it will be set. */
-    policy->nvPublic.nvPublic.attributes |= TPMA_NV_WRITTEN;
+    policy->nvPublic.attributes |= TPMA_NV_WRITTEN;
 
     /* Compute NV name from public info */
 
@@ -1204,6 +1198,95 @@ cleanup:
     return r;
 }
 
+/** Calculate a policy digest for policy template.
+ *
+ * The template hash will be derived from template_public if no template hash
+ * is provided.
+ *
+ * @param[in] policy The policy with the template hash or the public data used to
+ *            compute the template hash.
+ * @param[in,out] current_digest The digest list which has to be updated.
+ * @param[in] current_hash_alg The hash algorithm used for the policy computation.
+ *
+ * @retval TSS2_RC_SUCCESS on success.
+ * @retval TSS2_FAPI_RC_BAD_VALUE if an invalid value was passed into
+ *         the function.
+ * @retval TSS2_FAPI_RC_BAD_REFERENCE a invalid null pointer is passed.
+ * @retval TSS2_FAPI_RC_MEMORY if not enough memory can be allocated.
+ * @retval TSS2_FAPI_RC_GENERAL_FAILURE if an internal error occurred.
+ */
+TSS2_RC
+ifapi_calculate_policy_template(
+    TPMS_POLICYTEMPLATE *policy,
+    TPML_DIGEST_VALUES *current_digest,
+    TPMI_ALG_HASH current_hash_alg)
+
+{
+    TSS2_RC r = TSS2_RC_SUCCESS;
+    size_t digest_idx;
+    size_t hash_size;
+    TPM2B_DIGEST computed_template_hash;
+    TPM2B_DIGEST *used_template_hash;
+    IFAPI_CRYPTO_CONTEXT_BLOB *cryptoContext = NULL;
+    uint8_t buffer[sizeof(TPM2B_PUBLIC)];
+    size_t offset = 0;
+    size_t digest_size;
+
+    LOG_DEBUG("call");
+
+    if (!(hash_size = ifapi_hash_get_digest_size(current_hash_alg))) {
+        goto_error(r, TSS2_FAPI_RC_BAD_VALUE,
+                   "Unsupported hash algorithm (%" PRIu16 ")", cleanup,
+                   current_hash_alg);
+    }
+
+    /* Compute of the index of the current policy in the passed digest list */
+    r = get_policy_digest_idx(current_digest, current_hash_alg, &digest_idx);
+    return_if_error(r, "Get hash alg for digest.");
+
+    if (policy->templateHash.size == 0) {
+        used_template_hash = &computed_template_hash;
+        r = Tss2_MU_TPMT_PUBLIC_Marshal(&policy->templatePublic.publicArea,
+                                        &buffer[0], sizeof(TPMT_PUBLIC), &offset);
+        return_if_error(r, "Marshaling TPMT_PUBLIC");
+
+        r = ifapi_crypto_hash_start(&cryptoContext, current_hash_alg);
+        return_if_error(r, "crypto hash start");
+
+        HASH_UPDATE_BUFFER(cryptoContext,
+                           &buffer[0], offset,
+                           r, cleanup);
+        r = ifapi_crypto_hash_finish(&cryptoContext,
+                                     &used_template_hash->buffer[0],
+                                     &digest_size);
+        goto_if_error(r, "crypto hash finish", cleanup);
+        used_template_hash->size = digest_size;
+    } else {
+        used_template_hash = &policy->templateHash;
+    }
+
+    LOG_TRACE("Compute policy template");
+    r = ifapi_crypto_hash_start(&cryptoContext, current_hash_alg);
+    return_if_error(r, "crypto hash start");
+
+    HASH_UPDATE(cryptoContext, TPM2_CC, TPM2_CC_PolicyTemplate, r,
+                cleanup);
+    HASH_UPDATE_BUFFER(cryptoContext, &used_template_hash->buffer[0],
+                       used_template_hash->size, r, cleanup);
+    r = ifapi_crypto_hash_finish(&cryptoContext,
+                                 (uint8_t *) & current_digest->
+                                 digests[digest_idx].digest, &hash_size);
+    return_if_error(r, "crypto hash finish");
+
+    LOGBLOB_DEBUG((uint8_t *) & current_digest->digests[digest_idx].digest,
+                  hash_size, "Policy Duplicate digest");
+
+cleanup:
+    if (cryptoContext)
+        ifapi_crypto_hash_abort(&cryptoContext);
+    return r;
+}
+
 /** Compute policy digest for a list of policies.
  *
  * Every policy in the list will update the previous policy. Thus the final
@@ -1326,20 +1409,17 @@ ifapi_calculate_policy(
 
         case POLICYPHYSICALPRESENCE:
             r = ifapi_calculate_policy_physical_presence(
-                    &policy->elements[i].element.PolicyPhysicalPresence,
                     &policy->elements[i].policyDigests, hash_alg);
             return_if_error(r, "Compute policy physical presence");
             break;
 
         case POLICYAUTHVALUE:
-            r = ifapi_calculate_policy_auth_value(&policy->elements[i].element.PolicyAuthValue,
-                                                  &policy->elements[i].policyDigests, hash_alg);
+            r = ifapi_calculate_policy_auth_value(&policy->elements[i].policyDigests, hash_alg);
             return_if_error(r, "Compute policy auth value");
             break;
 
         case POLICYPASSWORD:
-            r = ifapi_calculate_policy_password(&policy->elements[i].element.PolicyPassword,
-                                                &policy->elements[i].policyDigests, hash_alg);
+            r = ifapi_calculate_policy_password(&policy->elements[i].policyDigests, hash_alg);
             return_if_error(r, "Compute policy password");
             break;
 
@@ -1371,6 +1451,15 @@ ifapi_calculate_policy(
             /* This does not alter the policyDigest */
             break;
 
+        case POLICYTEMPLATE:
+            r = ifapi_calculate_policy_template(&policy->elements[i].element.
+                                                 PolicyTemplate,
+                                                 &policy->elements[i].
+                                                 policyDigests, hash_alg);
+            return_if_error(r, "Compute policy template");
+
+            break;
+
         default:
             return_error(TSS2_FAPI_RC_BAD_VALUE,
                          "Policy not implemented");
