@@ -190,7 +190,7 @@ ifapi_get_object_name(
                                (TPM2B_NAME *)name);
             break;
         case IFAPI_NV_OBJ:
-            r = ifapi_nv_get_name(&object.misc.nv.public, name);
+            r = ifapi_nv_get_name(&object.misc.nv.public.nvPublic, name);
             break;
         default:
             goto_error(r, TSS2_FAPI_RC_BAD_VALUE, "Invalid object %s.",
@@ -243,7 +243,7 @@ TSS2_RC
 ifapi_get_nv_public(
     const char *path,
     TPMI_RH_NV_INDEX nv_index,
-    TPM2B_NV_PUBLIC *nv_public,
+    TPMS_NV_PUBLIC *nv_public,
     void *ctx)
 {
     TSS2_RC r = TSS2_RC_SUCCESS;
@@ -274,7 +274,7 @@ ifapi_get_nv_public(
                                               NULL);
                 try_again_or_error_goto(r, "Error: nv read public finish", cleanup);
 
-                *nv_public = *nv_public_esys;
+                *nv_public = nv_public_esys->nvPublic;
                 SAFE_FREE(nv_public_esys);
                 context->io_state = IO_INIT;
                 break;
@@ -302,7 +302,7 @@ ifapi_get_nv_public(
                                cleanup, path);
                 }
 
-                *nv_public = object.misc.nv.public;
+                *nv_public = object.misc.nv.public.nvPublic;
                 context->io_state = IO_INIT;
                 break;
 
@@ -320,10 +320,10 @@ cleanup:
 
 /** Read values of PCR registers and clear selection.
  *
- * @param[in,out] pcr_select The registers to be read (bank selection from profile).
- * @param[in,out] pcr_selection The registers to be read (with bank selection).
- * @param[out] pcr_values The callee-allocated public structure.
- * @param[in,out] ctx The context to access io and keystore module and to store
+ * @param[in] pcr_selection The registers to be read (with bank selection).
+ * @param[out] out_pcr_selection The registers that were read (with bank selection).
+ * @param[out] out_pcr_values The PCR values.
+ * @param[in] ctx The FAPI_CONTEXT to access io and keystore module and to store
  *                the io state.
  * @retval TSS2_RC_SUCCESS on success.
  * @retval TSS2_FAPI_RC_BAD_VALUE if the input parameters had inappropriate values.
@@ -333,29 +333,29 @@ cleanup:
  * @retval TSS2_FAPI_RC_BAD_SEQUENCE: if the context has an asynchronous
  *         operation already pending.
  */
-TSS2_RC
-ifapi_read_pcr(
-    TPMS_PCR_SELECT *pcr_select,
-    TPML_PCR_SELECTION *pcr_selection,
-    TPML_PCRVALUES **pcr_values,
-    void *ctx)
-{
+TSS2_RC ifapi_read_pcr(
+    TSS2_POLICY_PCR_SELECTION *pcr_selection,
+    TPML_PCR_SELECTION *out_pcr_selection,
+    TPML_DIGEST *out_pcr_digests,
+    void *ctx) {
+
     TSS2_RC r = TSS2_RC_SUCCESS;
     FAPI_CONTEXT *context = ctx;
     UINT32 update_counter;
     TPML_PCR_SELECTION *out_selection = NULL;
     TPML_PCR_SELECTION *profile_selection;
     TPML_DIGEST *pcr_digests = NULL;
-    size_t i, pcr, n_pcrs = 0, i_pcr;
+    size_t i, pcr;
+
+    TPML_PCR_SELECTION in_pcr_selection = { 0 };
 
     switch (context->io_state) {
     statecase(context->io_state, IO_INIT)
-        if (pcr_select->sizeofSelect) {
-            if (pcr_selection->count) {
-                /* If pcr_select is used pcr_selection can't be initialized */
-                return_error(TSS2_FAPI_RC_BAD_VALUE,
-                             "Policy PCR: pcr_selection can't be used if pcr_selection is used.");
-            }
+
+        /* Raw selector without bank specifier, figure out bank and copy
+         * the raw selector over to the input selector */
+        if (pcr_selection->type == TSS2_POLICY_PCR_SELECTOR_PCR_SELECT) {
+
             /* Determine hash alg */
             profile_selection = &context->profiles.default_profile.pcr_selection;
             for (i = 0; i < profile_selection->count; i++) {
@@ -363,28 +363,31 @@ ifapi_read_pcr(
                     uint8_t byte_idx = pcr / 8;
                     uint8_t flag = 1 << (pcr % 8);
                     /* Check whether PCR is used. */
-                    if (flag & profile_selection->pcrSelections[i].pcrSelect[byte_idx] &&
-                        flag & pcr_select->pcrSelect[byte_idx]) {
-                        pcr_selection->pcrSelections[0].hash = profile_selection->pcrSelections[i].hash;
+                    if ((flag & profile_selection->pcrSelections[i].pcrSelect[byte_idx]) &&
+                        (flag & pcr_selection->selections.pcr_select.pcrSelect[byte_idx])) {
+                        in_pcr_selection.pcrSelections[0].hash = profile_selection->pcrSelections[i].hash;
                     }
                 }
             }
-            if (!pcr_selection->pcrSelections[0].hash) {
+            if (!in_pcr_selection.pcrSelections[0].hash) {
                 /* hash for current pcr_select can't be determined */
                 return_error(TSS2_FAPI_RC_BAD_VALUE,
                              "Policy PCR: pcr_select does not match profile.");
             }
             /* Only one bank will be used. The hash alg from profile will be used */
-            pcr_selection->count = 1;
-            pcr_selection->pcrSelections[0].sizeofSelect = pcr_select->sizeofSelect;
-            for (i = 0; i < pcr_select->sizeofSelect; i++)
-                pcr_selection->pcrSelections[0].pcrSelect[i] = pcr_select->pcrSelect[i];
+            in_pcr_selection.count = 1;
+            in_pcr_selection.pcrSelections[0].sizeofSelect = pcr_selection->selections.pcr_select.sizeofSelect;
+            for (i = 0; i < pcr_selection->selections.pcr_select.sizeofSelect; i++)
+                in_pcr_selection.pcrSelections[0].pcrSelect[i] = pcr_selection->selections.pcr_select.pcrSelect[i];
+        } else {
+            /* we have specified selection with bank, so just copy it. */
+            in_pcr_selection = pcr_selection->selections.pcr_selection;
         }
 
         /* Prepare the PCR Reading. */
         r = Esys_PCR_Read_Async(context->esys,
                                 ESYS_TR_NONE, ESYS_TR_NONE, ESYS_TR_NONE,
-                                pcr_selection);
+                                &in_pcr_selection);
         return_if_error(r, "PCR Read");
         fallthrough;
 
@@ -397,41 +400,11 @@ ifapi_read_pcr(
 
         if (base_rc(r) == TSS2_BASE_RC_TRY_AGAIN)
             return TSS2_FAPI_RC_TRY_AGAIN;
+        goto_if_error2(r, "PCR_Read_Finish", cleanup);
 
-        return_if_error(r, "PCR_Read_Finish");
-
-        /* Count pcrs */
-        for (i = 0; i < out_selection->count; i++) {
-            for (pcr = 0; pcr < TPM2_MAX_PCRS; pcr++) {
-                uint8_t byte_idx = pcr / 8;
-                uint8_t flag = 1 << (pcr % 8);
-                /* Check whether PCR is used. */
-                if (flag & out_selection->pcrSelections[i].pcrSelect[byte_idx])
-                    n_pcrs += 1;
-            }
-        }
-
-        *pcr_values = calloc(1, sizeof(TPML_PCRVALUES) + n_pcrs* sizeof(TPMS_PCRVALUE));
-        goto_if_null2(*pcr_values, "Out of memory.", r, TSS2_FAPI_RC_MEMORY, cleanup);
-
-        /* Initialize digest list with pcr values from TPM */
-        i_pcr = 0;
-        (*pcr_values)->count = pcr_digests->count;
-        for (i = 0; i < out_selection->count; i++) {
-            for (pcr = 0; pcr < TPM2_MAX_PCRS; pcr++) {
-                uint8_t byte_idx = pcr / 8;
-                uint8_t flag = 1 << (pcr % 8);
-                /* Check whether PCR is used. */
-                if (flag & out_selection->pcrSelections[i].pcrSelect[byte_idx]) {
-                    (*pcr_values)->pcrs[i_pcr].pcr = pcr;
-                    (*pcr_values)->pcrs[i_pcr].hashAlg = out_selection->pcrSelections[i].hash;
-                    memcpy(&(*pcr_values)->pcrs[i_pcr].digest,
-                           &pcr_digests->digests[i_pcr].buffer[0],
-                           pcr_digests->digests[i_pcr].size);
-                    i_pcr += 1;
-                }
-            }
-        }
+        /* copy to caller */
+        *out_pcr_selection = *out_selection;
+        *out_pcr_digests = *pcr_digests;
 
         context->io_state = IO_INIT;
         break;
@@ -535,11 +508,11 @@ ifapi_policyeval_cbauth(
             if (cb_ctx->object.objectType == IFAPI_NV_OBJ) {
                 /* NV Authorization */
 
-                cb_ctx->nv_index = cb_ctx->object.handle;
+                cb_ctx->nv_index = cb_ctx->object.public.handle;
 
                 /* Determine the object used for authorization. */
                 get_nv_auth_object(&cb_ctx->object,
-                                   cb_ctx->object.handle,
+                                   cb_ctx->object.public.handle,
                                    &cb_ctx->auth_object,
                                    &cb_ctx->auth_index);
 
@@ -553,7 +526,7 @@ ifapi_policyeval_cbauth(
                 cb_ctx->auth_object_ptr = &cb_ctx->object;
                 return TSS2_FAPI_RC_TRY_AGAIN;
             } else {
-                cb_ctx->key_handle = cb_ctx->object.handle;
+                cb_ctx->key_handle = cb_ctx->object.public.handle;
                 if (cb_ctx->key_handle == ESYS_TR_NONE) {
                     cb_ctx->cb_state = POL_CB_LOAD_KEY;
                     return TSS2_FAPI_RC_TRY_AGAIN;
@@ -605,12 +578,13 @@ ifapi_policyeval_cbauth(
             fapi_ctx->loadKey = cb_ctx->load_ctx;
             fapi_ctx->createPrimary = cb_ctx->create_primary_ctx;
             cb_ctx->object = *cb_ctx->auth_object_ptr;
-            *auth_handle = cb_ctx->auth_object_ptr->handle;
+            *auth_handle = cb_ctx->auth_object_ptr->public.handle;
 
             r = ifapi_authorize_object(fapi_ctx, cb_ctx->auth_object_ptr, authSession);
             return_try_again(r);
-
             fapi_ctx->loadKey = cb_ctx->load_ctx_sav;
+            fapi_ctx->createPrimary = cb_ctx->create_primary_ctx;
+
             goto_if_error(r, "Authorize  object.", cleanup);
 
             cb_ctx->cb_state = POL_CB_EXECUTE_INIT;
@@ -619,11 +593,12 @@ ifapi_policyeval_cbauth(
 
         statecasedefault(cb_ctx->cb_state);
     }
-    *object_handle = cb_ctx->object.handle;
+
+    *object_handle = cb_ctx->object.public.handle;
     if (cb_ctx->object.objectType == IFAPI_NV_OBJ)
         *auth_handle = cb_ctx->auth_index;
     else
-        *auth_handle = cb_ctx->object.handle;
+        *auth_handle = cb_ctx->object.public.handle;
 
     if (current_policy->policySessionSav != ESYS_TR_NONE)
         fapi_ctx->policy.session = current_policy->policySessionSav;
@@ -636,6 +611,145 @@ cleanup:
     return r;
 }
 
+/** Callback for loading a key object used by policy.
+ *
+ * @param[in] name The name of the object to be loaded.
+ * @param[out] object_handle The ESYS handle of the used object.
+ * @param[in,out] userdata The Fapi context which will be used for keystore
+ *                access, and storing the policy execution state.
+ *                the io state.
+ * @retval TSS2_RC_SUCCESS on success.
+ * @retval TSS2_FAPI_RC_BAD_REFERENCE: if context or policy is NULL.
+ * @retval TSS2_FAPI_RC_MEMORY if memory allocation failed.
+ * @retval TSS2_FAPI_RC_TRY_AGAIN: if the asynchronous operation is not yet
+ *         complete. Call this function again later.
+ * @retval TSS2_FAPI_RC_BAD_SEQUENCE: if the context has an asynchronous
+ *         operation already pending.
+ * @retval TSS2_FAPI_RC_PATH_NOT_FOUND If a policy was not found.
+ * @retval TSS2_FAPI_RC_KEY_NOT_FOUND If a key was not found.
+ * @retval TSS2_FAPI_RC_IO_ERROR If an IO error occurred during reading
+ *         a policy or a key.
+ * @retval TSS2_FAPI_RC_GENERAL_FAILURE If an error in an used library
+ *         occurred.
+ * @retval TSS2_FAPI_RC_BAD_VALUE if an invalid value was passed into
+ *         the function.
+ * @retval TSS2_FAPI_RC_AUTHORIZATION_UNKNOWN if a required authorization callback
+ *         is not set.
+ * @retval TSS2_FAPI_RC_AUTHORIZATION_FAILED if the authorization attempt fails.
+ * @retval TSS2_FAPI_RC_POLICY_UNKNOWN if policy search for a certain policy digest
+ *         was not successful.
+ * @retval TSS2_ESYS_RC_* possible error codes of ESAPI.
+ * @retval TSS2_FAPI_RC_BAD_PATH if the path is used in inappropriate context
+ *         or contains illegal characters.
+ * @retval TSS2_FAPI_RC_NOT_PROVISIONED FAPI was not provisioned.
+ */
+TSS2_RC
+ifapi_policyeval_cbload_key(
+    TPM2B_NAME *name,
+    ESYS_TR *object_handle,
+    void *userdata)
+{
+    TSS2_RC r;
+    FAPI_CONTEXT *fapi_ctx = userdata;
+    IFAPI_POLICY_EXEC_CTX *current_policy;
+    IFAPI_POLICY_EXEC_CB_CTX *cb_ctx;
+
+    return_if_null(fapi_ctx, "Bad user data.", TSS2_FAPI_RC_BAD_REFERENCE);
+    return_if_null(fapi_ctx->policy.policyutil_stack, "Policy not initialized.",
+                   TSS2_FAPI_RC_BAD_REFERENCE);
+
+    if (fapi_ctx->policy.util_current_policy) {
+        /* Use the current policy in the policy stack. */
+        current_policy = fapi_ctx->policy.util_current_policy->pol_exec_ctx;
+    } else {
+        /* Start with the bottom of the policy stack */
+        current_policy = fapi_ctx->policy.policyutil_stack->pol_exec_ctx;
+    }
+    cb_ctx = current_policy->app_data;
+
+    switch (cb_ctx->cb_state) {
+        statecase(cb_ctx->cb_state, POL_CB_EXECUTE_INIT);
+            cb_ctx->flush_handle = false;
+            cb_ctx->auth_index = ESYS_TR_NONE;
+            /* Search object with name in keystore. */
+            r = ifapi_keystore_search_obj(&fapi_ctx->keystore, &fapi_ctx->io,
+                                          name,
+                                          &cb_ctx->object_path);
+            FAPI_SYNC(r, "Search Object", cleanup);
+
+            r = ifapi_keystore_load_async(&fapi_ctx->keystore, &fapi_ctx->io,
+                                          cb_ctx->object_path);
+            return_if_error2(r, "Could not open: %s", cb_ctx->object_path);
+            SAFE_FREE(cb_ctx->object_path);
+            fallthrough;
+
+        statecase(cb_ctx->cb_state, POL_CB_READ_OBJECT);
+            /* Get object from file */
+            r = ifapi_keystore_load_finish(&fapi_ctx->keystore, &fapi_ctx->io,
+                                           &cb_ctx->object);
+            return_try_again(r);
+            return_if_error(r, "read_finish failed");
+
+            r = ifapi_initialize_object(fapi_ctx->esys, &cb_ctx->object);
+            goto_if_error(r, "Initialize key", cleanup);
+
+            if (cb_ctx->object.objectType != IFAPI_KEY_OBJ) {
+                goto_error(r, TSS2_FAPI_RC_BAD_VALUE,
+                           "Key object expected", cleanup);
+            }
+            cb_ctx->key_handle = cb_ctx->object.public.handle;
+            if (cb_ctx->key_handle != ESYS_TR_NONE) {
+                break;
+            }
+            fallthrough;
+
+        statecase(cb_ctx->cb_state, POL_CB_LOAD_KEY);
+            /* Prepare new context for loadkey in policy and skip session creation. */
+            memset(&cb_ctx->load_ctx, 0, sizeof(IFAPI_LoadKey));
+            cb_ctx->load_ctx.prepare_state = PREPARE_LOAD_KEY_INIT_KEY;
+            memset(&cb_ctx->create_primary_ctx, 0, sizeof(IFAPI_CreatePrimary));
+            fallthrough;
+
+        statecase(cb_ctx->cb_state, POL_CB_LOAD_KEY_FINISH);
+            cb_ctx->load_ctx_sav = fapi_ctx->loadKey;
+            cb_ctx->create_primary_ctx_sav = fapi_ctx->createPrimary;
+            fapi_ctx->loadKey = cb_ctx->load_ctx;
+            fapi_ctx->createPrimary = cb_ctx->create_primary_ctx;
+            cb_ctx->auth_object_ptr = &cb_ctx->load_ctx.auth_object;
+            r = ifapi_load_key(fapi_ctx, ifapi_get_object_path(&cb_ctx->object),
+                               &cb_ctx->auth_object_ptr);
+            if (r == TSS2_RC_SUCCESS &&
+                !cb_ctx->load_ctx.auth_object.misc.key.persistent_handle) {
+                current_policy->flush_handle = true;
+            }
+            cb_ctx->load_ctx = fapi_ctx->loadKey;
+            cb_ctx->create_primary_ctx = fapi_ctx->createPrimary;
+            fapi_ctx->loadKey = cb_ctx->load_ctx_sav;
+            fapi_ctx->createPrimary = cb_ctx->create_primary_ctx_sav;
+            FAPI_SYNC(r, "Fapi load key.", cleanup);
+
+            ifapi_cleanup_ifapi_object(&cb_ctx->object);
+            cb_ctx->object = *cb_ctx->auth_object_ptr;
+
+            *object_handle = cb_ctx->auth_object_ptr->public.handle;
+
+            cb_ctx->cb_state = POL_CB_EXECUTE_INIT;
+            break;
+
+       statecasedefault(cb_ctx->cb_state);
+    }
+
+    if (current_policy->policySessionSav != ESYS_TR_NONE)
+        fapi_ctx->policy.session = current_policy->policySessionSav;
+
+cleanup:
+     ifapi_cleanup_ifapi_object(&cb_ctx->object);
+    if (current_policy->policySessionSav
+        && current_policy->policySessionSav != ESYS_TR_NONE)
+        fapi_ctx->policy.session = current_policy->policySessionSav;
+    return r;
+}
+
 /** Callback for branch selection of policy or.
  *
  * @param[in]  branches The list of policy branches.
@@ -653,42 +767,32 @@ cleanup:
  */
 TSS2_RC
 ifapi_branch_selection(
-    TPML_POLICYBRANCHES *branches,
+    TSS2_OBJECT *auth_object,
+    const char *branch_names[8],
+    size_t branch_count,
     size_t *branch_idx,
     void *userdata)
 {
     TSS2_RC r;
     FAPI_CONTEXT *fapi_ctx = userdata;
-    size_t i;
-    const char *names[8];
-    IFAPI_OBJECT *auth_object;
     const char* object_path;
 
     return_if_null(fapi_ctx, "Bad user data.", TSS2_FAPI_RC_BAD_REFERENCE);
 
-    if (!fapi_ctx->callbacks.branch) {
-        return_error(TSS2_FAPI_RC_AUTHORIZATION_UNKNOWN,
-                     "No branch selection callback");
-    }
-    for (i = 0; i < branches->count; i++)
-        names[i] = branches->authorizations[i].name;
-
     /* Determine path of object to be authenticated. */
-    auth_object = fapi_ctx->policy.util_current_policy->pol_exec_ctx->auth_object;
     return_if_null(auth_object, "No object passed.", TSS2_FAPI_RC_BAD_REFERENCE);
 
-    object_path = ifapi_get_object_path(auth_object);
+    return_if_null(fapi_ctx->callbacks.branch, "No branch selection callback", TSS2_FAPI_RC_AUTHORIZATION_UNKNOWN);
+
+    object_path = ifapi_get_object_path(TSS2_OBJECT_TO_IFAPI_OBJECT(auth_object));
 
     r = fapi_ctx->callbacks.branch(object_path, "PolicyOR",
-                                   &names[0],
-                                   branches->count,
+                                   &branch_names[0],
+                                   branch_count,
                                    branch_idx,
                                    fapi_ctx->callbacks.branchData);
     return_if_error(r, "policyBranchSelectionCallback");
 
-    if (*branch_idx >= branches->count) {
-        return_error2(TSS2_FAPI_RC_AUTHORIZATION_FAILED, "Invalid branch number.");
-    }
     return TSS2_RC_SUCCESS;
 }
 
@@ -848,7 +952,7 @@ equal_policy_authorization(
                     pem_public.publicArea.parameters.rsaDetail.scheme
                         = authorizations->authorizations[i].rsaScheme;
                 }
-                pem_public.publicArea.nameAlg = authorizations->authorizations[i].keyPEMhashAlg;
+                pem_public.publicArea.nameAlg = authorizations->authorizations[i].hashAlg;
 
                 /* Check public information if key and policyRef */
                 if (ifapi_TPMT_PUBLIC_cmp(public, &pem_public.publicArea) &&
@@ -1139,13 +1243,13 @@ get_policy_signature(
                 pem_public.publicArea.parameters.rsaDetail.scheme
                     = authorizations->authorizations[i].rsaScheme;
             }
-            pem_public.publicArea.nameAlg = authorizations->authorizations[i].keyPEMhashAlg;
+            pem_public.publicArea.nameAlg = authorizations->authorizations[i].hashAlg;
 
             if (ifapi_TPMT_PUBLIC_cmp(public, &pem_public.publicArea)) {
                 r = ifapi_der_sig_to_tpm(&pem_public.publicArea,
                                          authorizations->authorizations[i].pemSignature.buffer,
                                          authorizations->authorizations[i].pemSignature.size,
-                                         authorizations->authorizations[i].keyPEMhashAlg,
+                                         authorizations->authorizations[i].hashAlg,
                                          signature);
 
                 return_if_error(r, "Invalid signature.");
@@ -1402,7 +1506,7 @@ cleanup:
  */
 TSS2_RC
 ifapi_exec_auth_nv_policy(
-    TPM2B_NV_PUBLIC *nv_public,
+    TPMS_NV_PUBLIC *nv_public,
     TPMI_ALG_HASH hash_alg,
     void *userdata)
 {
@@ -1437,9 +1541,14 @@ ifapi_exec_auth_nv_policy(
 
     switch (cb_ctx->cb_state) {
         statecase(cb_ctx->cb_state, POL_CB_EXECUTE_INIT)
-            /* Search a NV object with a certain NV indext stored in nv_public. */
+            /*
+             * Search a NV object with a certain NV indext stored in nv_public.
+             * TODO Consider modifying internal APIs to use TPMS_NV_INDEX over TPM2B variant.
+             */
+            TPM2B_NV_PUBLIC tmp = { 0 };
+            tmp.nvPublic = *nv_public;
             r = ifapi_keystore_search_nv_obj(&fapi_ctx->keystore, &fapi_ctx->io,
-                                             nv_public, &nv_path);
+                                             &tmp, &nv_path);
             FAPI_SYNC(r, "Search Object", cleanup);
 
             r = ifapi_keystore_load_async(&fapi_ctx->keystore, &fapi_ctx->io, nv_path);
@@ -1458,7 +1567,7 @@ ifapi_exec_auth_nv_policy(
             r = ifapi_initialize_object(esys_ctx, &cb_ctx->object);
             goto_if_error(r, "Initialize NV object", cleanup);
 
-            current_policy->nv_index = cb_ctx->object.handle;
+            current_policy->nv_index = cb_ctx->object.public.handle;
             ifapi_cleanup_ifapi_object(&cb_ctx->object);
             get_nv_auth_object(&cb_ctx->object,
                                current_policy->nv_index,
