@@ -1,5 +1,8 @@
 /* SPDX-License-Identifier: BSD-2-Clause */
-
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+#include <assert.h>
 #include <stdarg.h>
 #include <stdbool.h>
 #include <stdio.h>
@@ -9,8 +12,6 @@
 #include "tss2_sys.h"
 #include "util/aux_util.h"
 
-#define ARRAY_LEN(x) (sizeof(x)/sizeof(x[0]))
-
 /**
  * The maximum size of a layer name.
  */
@@ -195,8 +196,8 @@ tpm2_rc_fmt0_S_get(TSS2_RC rc)
  */
 #define ADD_NULL_HANDLER ADD_HANDLER("\0", NULL)
 
-static const char *
-tpm2_err_handler_fmt1(TPM2_RC rc)
+const char *
+tss2_fmt1_err_strs_get(TSS2_RC error)
 {
     /*
      * format 1 error codes start at 1, so
@@ -290,45 +291,15 @@ tpm2_err_handler_fmt1(TPM2_RC rc)
         "point is not on the required curve",
     };
 
-    static __thread char buf[TSS2_ERR_LAYER_ERROR_STR_MAX + 1];
-
-    clearbuf(buf);
-
-    /* Print whether or not the error is caused by a bad
-     * handle or parameter. On the case of a Handle (P == 0)
-     * then the N field top bit will be set. Un-set this bit
-     * to get the handle index by subtracting 8 as N is a 4
-     * bit field.
-     *
-     * the lower 3 bits of N indicate index, and the high bit
-     * indicates
-     */
-    UINT8 index = tpm2_rc_fmt1_N_index_get(rc);
-
-    bool is_handle = tpm2_rc_fmt1_N_is_handle(rc);
-    const char *m = tpm2_rc_fmt1_P_get(rc) ? "parameter" :
-                    is_handle ? "handle" : "session";
-    catbuf(buf, "%s", m);
-
-    if (index) {
-        catbuf(buf, "(%u):", index);
-    } else {
-        catbuf(buf, "%s", "(unk):");
+    if (error < ARRAY_LEN(fmt1_err_strs)) {
+        return fmt1_err_strs[error];
     }
 
-    UINT8 errnum = tpm2_rc_fmt1_error_get(rc);
-    if (errnum < ARRAY_LEN(fmt1_err_strs)) {
-        m = fmt1_err_strs[errnum];
-        catbuf(buf, "%s", m);
-    } else {
-        catbuf(buf, "unknown error num: 0x%X", errnum);
-    }
-
-    return buf;
+    return NULL;
 }
 
-static const char *
-tpm2_err_handler_fmt0(TSS2_RC rc)
+const char *
+tss2_fmt0_err_strs_get(TSS2_RC rc)
 {
     /*
      * format 0 error codes start at 1, so
@@ -625,6 +596,60 @@ tpm2_err_handler_fmt0(TSS2_RC rc)
         "the sensitive area did not unmarshal correctly after decryption",
     };
 
+    UINT8 errnum = tpm2_rc_fmt0_error_get(rc);
+    /* is it a warning (version 2 error string) or is it a 1.2 error? */
+    size_t len = tpm2_rc_fmt0_S_get(rc) ? ARRAY_LEN(fmt0_warn_strs) : ARRAY_LEN(fmt0_err_strs);
+    const char **selection = tpm2_rc_fmt0_S_get(rc) ? fmt0_warn_strs : fmt0_err_strs;
+    if (errnum >= len) {
+        return NULL;
+    }
+
+    return selection[errnum];
+}
+
+static const char *
+tpm2_err_handler_fmt1(TPM2_RC rc)
+{
+    static __thread char buf[TSS2_ERR_LAYER_ERROR_STR_MAX + 1];
+
+    clearbuf(buf);
+
+    /* Print whether or not the error is caused by a bad
+     * handle or parameter. On the case of a Handle (P == 0)
+     * then the N field top bit will be set. Un-set this bit
+     * to get the handle index by subtracting 8 as N is a 4
+     * bit field.
+     *
+     * the lower 3 bits of N indicate index, and the high bit
+     * indicates
+     */
+    UINT8 index = tpm2_rc_fmt1_N_index_get(rc);
+
+    bool is_handle = tpm2_rc_fmt1_N_is_handle(rc);
+    const char *m = tpm2_rc_fmt1_P_get(rc) ? "parameter" :
+                    is_handle ? "handle" : "session";
+    catbuf(buf, "%s", m);
+
+    if (index) {
+        catbuf(buf, "(%u):", index);
+    } else {
+        catbuf(buf, "%s", "(unk):");
+    }
+
+    UINT8 errnum = tpm2_rc_fmt1_error_get(rc);
+    m = tss2_fmt1_err_strs_get(errnum);
+    if (m) {
+        catbuf(buf, "%s", m);
+    } else {
+        catbuf(buf, "unknown error num: 0x%X", errnum);
+    }
+
+    return buf;
+}
+
+static const char *
+tpm2_err_handler_fmt0(TSS2_RC rc)
+{
     static __thread char buf[TSS2_ERR_LAYER_ERROR_STR_MAX + 1];
 
     clearbuf(buf);
@@ -642,17 +667,7 @@ tpm2_err_handler_fmt0(TSS2_RC rc)
             return buf;
         }
 
-        /* is it a warning (version 2 error string) or is it a 1.2 error? */
-        size_t len =
-                tpm2_rc_fmt0_S_get(rc) ?
-                        ARRAY_LEN(fmt0_warn_strs) : ARRAY_LEN(fmt0_err_strs);
-        const char **selection =
-                tpm2_rc_fmt0_S_get(rc) ? fmt0_warn_strs : fmt0_err_strs;
-        if (errnum >= len) {
-            return NULL;
-        }
-
-        const char *m = selection[errnum];
+        const char *m = tss2_fmt0_err_strs_get(rc);
         if (!m) {
             return NULL;
         }
@@ -834,7 +849,7 @@ tss_err_handler (TSS2_RC rc)
 static struct {
     char name[TSS2_ERR_LAYER_NAME_MAX];
     TSS2_RC_HANDLER handler;
-} layer_handler[TPM2_ERROR_TSS2_RC_LAYER_COUNT] = {
+} layer_handler[TPM2_ERROR_TSS2_RC_LAYER_COUNT + 1] = {
     ADD_HANDLER("tpm" , tpm2_ehandler),
     ADD_NULL_HANDLER,                       /* layer 1  is unused */
     ADD_NULL_HANDLER,                       /* layer 2  is unused */
@@ -852,7 +867,7 @@ static struct {
                                             /* The RM usually duplicates TPM responses */
                                             /* So just default the handler to tpm2. */
     ADD_HANDLER("rm", NULL),                /* layer 12 is the rm rc */
-    ADD_HANDLER("drvr", NULL),              /* layer 13 is the driver rc */
+    ADD_HANDLER("policy", tss_err_handler), /* layer 13 is the policy rc */
 };
 
 /**
@@ -869,7 +884,7 @@ unknown_layer_handler(TSS2_RC rc)
     static __thread char buf[32];
 
     clearbuf(buf);
-    catbuf(buf, "0x%X", tpm2_error_get(rc));
+    catbuf(buf, "0x%X", rc);
 
     return buf;
 }
@@ -966,19 +981,102 @@ Tss2_RC_Decode(TSS2_RC rc)
         catbuf(buf, "%u:", layer);
     }
 
-    handler = !handler ? unknown_layer_handler : handler;
-
     /*
      * Handlers only need the error bits. This way they don't
      * need to concern themselves with masking off the layer
      * bits or anything else.
      */
-    UINT16 err_bits = tpm2_error_get(rc);
-    const char *e = err_bits ? handler(err_bits) : "success";
-    if (e) {
-        catbuf(buf, "%s", e);
+    if (handler) {
+        UINT16 err_bits = tpm2_error_get(rc);
+        const char *e = err_bits ? handler(err_bits) : "success";
+        if (e) {
+            catbuf(buf, "%s", e);
+        } else {
+            catbuf(buf, "0x%X", err_bits);
+        }
     } else {
-        catbuf(buf, "0x%X", err_bits);
+        /*
+         * we don't want to drop any bits if we don't know what to do with it
+         * so drop the layer byte since we we already have that.
+         */
+        const char *e = unknown_layer_handler(rc >> 8);
+        assert(e);
+        catbuf(buf, "%s", e);
+    }
+
+    return buf;
+}
+
+/** Function to extract information from a response code.
+ *
+ * This function decodes the different bitfields in TSS2_RC.
+ *
+ * @param[in]  rc the response code to decode.
+ * @param[out] info the structure containing the decoded fields.
+ * @retval TSS2_RC_SUCCESS if the function call was a success.
+ * @retval TSS2_ESYS_RC_BAD_REFERENCE if info is a NULL pointer.
+ */
+TSS2_RC
+Tss2_RC_DecodeInfo(TSS2_RC rc, TSS2_RC_INFO *info)
+{
+    UINT8 n;
+
+    if (!info) {
+        return TSS2_BASE_RC_BAD_REFERENCE;
+    }
+
+    memset(info, 0, sizeof(TSS2_RC_INFO));
+
+    info->layer = tss2_rc_layer_number_get(rc);
+    info->format = tss2_rc_layer_format_get(rc);
+
+    if (info->format) {
+        info->error = tpm2_rc_fmt1_error_get(rc) | TPM2_RC_FMT1;
+        n = tpm2_rc_fmt1_N_index_get(rc);
+        if (tpm2_rc_fmt1_P_get(rc)) {
+	    info->parameter = n;
+        } else if (tpm2_rc_fmt1_N_is_handle(rc)) {
+            info->handle = n;
+        } else {
+          info->session = n;
+        }
+    } else {
+        info->error = tpm2_error_get(rc);
+    }
+
+    return TSS2_RC_SUCCESS;
+}
+
+/** Function to get a human readable error from a TSS2_RC_INFO
+ *
+ * This function returns the human readable eror for the underlying
+ * error, ignoring the layer, parameters, handles and sessions.
+ *
+ * @param[int] info the structure containing the decoded fields.
+ * @retval A human understandable error description string.
+ * @retval NULL if info is a NULL pointer.
+ */
+const char *
+Tss2_RC_DecodeInfoError(TSS2_RC_INFO *info)
+{
+    static __thread char buf[TSS2_ERR_LAYER_ERROR_STR_MAX + 1];
+    const char *m = NULL;
+
+    if (!info) {
+        return NULL;
+    }
+    clearbuf(buf);
+
+    if (info->format) {
+        m = tss2_fmt1_err_strs_get(info->error ^ TPM2_RC_FMT1);
+    } else {
+        m = tss2_fmt0_err_strs_get(info->error ^ TPM2_RC_VER1);
+    }
+
+    if (m) {
+        catbuf(buf, "%s", m);
+    } else {
+        catbuf(buf, "0x%X", info->error);
     }
 
     return buf;
