@@ -11,6 +11,8 @@
 
 #include <errno.h>
 #include <inttypes.h>
+#include <stddef.h>
+#include <stdlib.h>
 #if defined(__linux__)
 #include <linux/limits.h>
 #elif defined(_MSC_VER)
@@ -74,9 +76,22 @@ tcti_from_init(TSS2_TCTI_INIT_FUNC init,
         return TSS2_ESYS_RC_MEMORY;
     }
 
+    /* Unless tcti loglevel is log_debug or higher
+     * (i.e. TSS2_LOG=tcti+debug) turn the logging
+     * from loaded tctis off completely, including warnings
+     * and error logs. It makes too much noise when tcti
+     * loader tries them all one by one and what we want
+     * use is the last one.
+     */
+    log_level old_loglevel = LOGMODULE_status;
+    if (LOGMODULE_status < LOGLEVEL_INFO)
+        LOGMODULE_status = LOGLEVEL_NONE;
+
     r = init(*tcti, &size, conf);
+    LOGMODULE_status = old_loglevel;
+
     if (r != TSS2_RC_SUCCESS) {
-        LOG_WARNING("TCTI init for function %p failed with %" PRIx32, init, r);
+        LOG_DEBUG("TCTI init for function %p failed with %" PRIx32, init, r);
         free(*tcti);
         *tcti=NULL;
         return r;
@@ -106,11 +121,10 @@ tcti_from_info(TSS2_TCTI_INFO_FUNC infof,
 
     r = tcti_from_init(info->init, conf, tcti);
     if (r != TSS2_RC_SUCCESS) {
-        LOG_WARNING("Could not initialize TCTI named: %s", info->name);
+        LOG_DEBUG("Could not initialize TCTI named: %s", info->name);
         return r;
     }
-
-    LOG_DEBUG("Initialized TCTI named: %s", info->name);
+    LOG_INFO("Initialized TCTI named: %s", info->name);
 
     return TSS2_RC_SUCCESS;
 }
@@ -448,14 +462,32 @@ TSS2_RC
 Tss2_TctiLdr_Initialize (const char *nameConf,
                          TSS2_TCTI_CONTEXT **tctiContext)
 {
-    char name [PATH_MAX] = { 0, }, conf [PATH_MAX] = { 0, };
+    char *name = NULL;
+    char *conf = NULL;
     TSS2_RC rc;
+    size_t combined_length;
 
     if (nameConf == NULL) {
         return Tss2_TctiLdr_Initialize_Ex (NULL, NULL, tctiContext);
     }
+
+    combined_length = strlen (nameConf);
+    if (combined_length > PATH_MAX - 1) {
+        LOG_ERROR ("combined conf length must be between 0 and PATH_MAX");
+        return TSS2_TCTI_RC_BAD_VALUE;
+    }
+    name = calloc(combined_length + 1, sizeof(char));
+    conf = calloc(combined_length + 1, sizeof(char));
+    if (name == NULL || conf == NULL) {
+        rc = TSS2_TCTI_RC_MEMORY;
+        goto out;
+    }
     rc = tctildr_conf_parse (nameConf, name, conf);
     if (rc != TSS2_RC_SUCCESS)
-        return rc;
-    return Tss2_TctiLdr_Initialize_Ex (name, conf, tctiContext);
+        goto out;
+    rc = Tss2_TctiLdr_Initialize_Ex (name, conf, tctiContext);
+out:
+    SAFE_FREE(name);
+    SAFE_FREE(conf);
+    return rc;
 }
