@@ -8,12 +8,14 @@
 #endif
 
 #include <errno.h>
+#include <fcntl.h>
 #include <inttypes.h>
 #include <limits.h>
 #include <string.h>
 #include <stdio.h>
 
 #ifndef _WIN32
+#include <poll.h>
 #include <netdb.h>
 #include <netinet/in.h>
 #include <unistd.h>
@@ -26,6 +28,12 @@
 #include "util/log.h"
 
 #define MAX_PORT_STR_LEN    sizeof("65535")
+
+/* sockaddr_un::sun_path is documented as char[108], but it seems safer to let
+ * the compiler (or rather, the headers) derive this for us. (Cast to int to
+ * avoid signed/unsigned comparison warnings.) */
+#define MAX_SADDR_UN_PATH  (int)sizeof(((struct sockaddr_un *)0)->sun_path)
+
 /*
  * The 'read_all' function attempts to read all of the 'size' bytes requested
  * from the 'fd' provided into the buffer 'data'. This function will continue
@@ -179,6 +187,7 @@ TSS2_RC
 socket_connect (
     const char *hostname,
     uint16_t port,
+    int control,
     SOCKET *sock)
 {
     static const struct addrinfo hints = { .ai_socktype = SOCK_STREAM,
@@ -206,6 +215,9 @@ socket_connect (
         return TSS2_TCTI_RC_BAD_REFERENCE;
     }
 
+    if (control)
+        port++;
+
     ret = snprintf(port_str, sizeof(port_str), "%u", port);
     if (ret < 0)
         return TSS2_TCTI_RC_BAD_VALUE;
@@ -236,8 +248,7 @@ socket_connect (
         if (h == NULL)
             h = hostname;
 
-        LOG_DEBUG ("Attempting TCP connection to host %s, port %s",
-            h, port_str);
+        LOG_DEBUG ("Attempting TCP connection to host %s, port %s", h, port_str);
         if (connect (*sock, p->ai_addr, p->ai_addrlen) != SOCKET_ERROR)
             break; /* socket connected OK */
         socket_close (sock);
@@ -257,3 +268,92 @@ socket_connect (
 
     return TSS2_RC_SUCCESS;
 }
+
+TSS2_RC
+socket_connect_unix (
+    const char *path,
+    int control,
+    SOCKET *sock)
+{
+#ifdef _WIN32
+    return TSS2_TCTI_RC_BAD_REFERENCE;
+#else
+    struct sockaddr_un saddr;
+
+    if (path == NULL)
+        return TSS2_TCTI_RC_BAD_REFERENCE;
+
+    saddr.sun_family = AF_UNIX;
+
+    if (snprintf(saddr.sun_path, MAX_SADDR_UN_PATH,
+                 control ? "%s.ctrl" : "%s", path) >= MAX_SADDR_UN_PATH) {
+        LOG_ERROR ("Socket %s%s is too long for AF_UNIX",
+                   path, control ? ".ctrl" : "");
+        return TSS2_TCTI_RC_BAD_VALUE;
+    }
+
+    *sock = socket (AF_UNIX, SOCK_STREAM, 0);
+
+    if (*sock == INVALID_SOCKET) {
+        LOG_WARNING ("Failed to create AF_UNIX socket");
+        return TSS2_TCTI_RC_IO_ERROR;
+    }
+
+    LOG_DEBUG ("Attempting UNIX connection to %s", saddr.sun_path);
+    if (connect (*sock, (struct sockaddr *)&saddr, sizeof(saddr)) == SOCKET_ERROR) {
+        LOG_WARNING ("Failed to connect to %s", saddr.sun_path);
+        return TSS2_TCTI_RC_IO_ERROR;
+    }
+
+    return TSS2_RC_SUCCESS;
+#endif
+}
+
+TSS2_RC
+socket_set_nonblock (SOCKET sock)
+{
+#ifndef _WIN32
+    int flgs;
+
+    flgs = fcntl(sock, F_GETFL);
+    if (flgs == -1)
+        return TSS2_TCTI_RC_IO_ERROR;
+
+    flgs |= O_NONBLOCK;
+    if (fcntl(sock, F_SETFL, flgs) != 0)
+        return TSS2_TCTI_RC_IO_ERROR;
+#endif
+    return TSS2_RC_SUCCESS;
+}
+
+TSS2_RC
+socket_poll (SOCKET sock, int timeout)
+{
+#ifndef _WIN32
+    struct pollfd fds;
+    int rc_poll, nfds = 1;
+
+    fds.fd = sock;
+    fds.events = POLLIN;
+
+    /* Timeout of 0 ie return immediately is not
+     * well handled throughout the upper layers currenty
+     * cousing the integration tests to hang randomly.
+     * Make the poll to waint to at least 10 ms */
+    if (timeout == 0)
+        timeout = 10;
+
+    rc_poll = poll(&fds, nfds, timeout);
+    if (rc_poll < 0) {
+        LOG_ERROR ("Failed to poll for response from fd %d, got errno %d: %s",
+                   sock, errno, strerror(errno));
+        return TSS2_TCTI_RC_IO_ERROR;
+    } else if (rc_poll == 0) {
+        LOG_INFO ("Poll timed out on fd %d.", sock);
+        return TSS2_TCTI_RC_TRY_AGAIN;
+    } else if (fds.revents == POLLIN) {
+        return TSS2_RC_SUCCESS;
+    }
+#endif
+    return TSS2_RC_SUCCESS;
+}
