@@ -30,7 +30,7 @@
  * Secret exchange with a ECC key can be activated with the compiler variable
  * -D TEST_ECC.
  *
- * Tested ESAPI commands:
+ * Tested ESYS commands:
  *  - Esys_ContextLoad() (M)
  *  - Esys_ContextSave() (M)
  *  - Esys_Create() (M)
@@ -53,14 +53,41 @@ test_esys_create_session_auth(ESYS_CONTEXT * esys_context)
     TSS2_RC r;
     ESYS_TR primaryHandle = ESYS_TR_NONE;
     ESYS_TR loadedKeyHandle = ESYS_TR_NONE;
+#if defined(TEST_ECC) || !defined(TEST_NULL_BIND_NO_TPM_KEY)
     ESYS_TR primaryHandle_AuthSession = ESYS_TR_NONE;
+#endif
     ESYS_TR session = ESYS_TR_NONE;
+    ESYS_TR outerSession = ESYS_TR_NONE;
+
+    TPM2B_PUBLIC *outPublic = NULL;
+    TPM2B_CREATION_DATA *creationData = NULL;
+    TPM2B_DIGEST *creationHash = NULL;
+    TPMT_TK_CREATION *creationTicket = NULL;
+
+#ifdef TEST_ECC
+    TPM2B_PUBLIC *outPublicEcc = NULL;
+    TPM2B_CREATION_DATA *creationDataEcc = NULL;
+    TPM2B_DIGEST *creationHashEcc = NULL;
+    TPMT_TK_CREATION *creationTicketEcc = NULL;
+#endif
+
+    TPM2B_PUBLIC *outPublic2 = NULL;
+    TPM2B_PRIVATE *outPrivate2 = NULL;
+    TPM2B_CREATION_DATA *creationData2 = NULL;
+    TPM2B_DIGEST *creationHash2 = NULL;
+    TPMT_TK_CREATION *creationTicket2 = NULL;
 
     TPM2B_AUTH authValuePrimary = {
         .size = 5,
         .buffer = {1, 2, 3, 4, 5}
     };
 
+#ifdef TEST_LARGE_AUTH
+    for (int i = 0; i < 33; i++)
+        authValuePrimary.buffer[i] = i;
+    authValuePrimary.size = 33;
+#endif
+
     TPM2B_SENSITIVE_CREATE inSensitivePrimary = {
         .size = 0,
         .sensitive = {
@@ -100,7 +127,7 @@ test_esys_create_session_auth(ESYS_CONTEXT * esys_context)
                  .scheme = {
                       .scheme = TPM2_ALG_ECDH,
                       .details = {
-                          .ecdh = {.hashAlg  = TPM2_ALG_SHA1}},
+                          .ecdh = {.hashAlg  = TPM2_ALG_SHA256}},
                   },
                  .curveID = TPM2_ECC_NIST_P256,
                  .kdf = {
@@ -165,10 +192,6 @@ test_esys_create_session_auth(ESYS_CONTEXT * esys_context)
     goto_if_error(r, "Error: TR_SetAuth", error);
 
     RSRC_NODE_T *primaryHandle_node;
-    TPM2B_PUBLIC *outPublic;
-    TPM2B_CREATION_DATA *creationData;
-    TPM2B_DIGEST *creationHash;
-    TPMT_TK_CREATION *creationTicket;
 
     r = Esys_CreatePrimary(esys_context, ESYS_TR_RH_OWNER, ESYS_TR_PASSWORD,
                            ESYS_TR_NONE, ESYS_TR_NONE, &inSensitivePrimary, &inPublic,
@@ -189,11 +212,6 @@ test_esys_create_session_auth(ESYS_CONTEXT * esys_context)
 
 
 #ifdef TEST_ECC
-    TPM2B_PUBLIC *outPublicEcc;
-    TPM2B_CREATION_DATA *creationDataEcc;
-    TPM2B_DIGEST *creationHashEcc;
-    TPMT_TK_CREATION *creationTicketEcc;
-
     r = Esys_CreatePrimary(esys_context, ESYS_TR_RH_OWNER, ESYS_TR_PASSWORD,
                            ESYS_TR_NONE, ESYS_TR_NONE, &inSensitivePrimary, &inPublicEcc,
                            &outsideInfo, &creationPCR, &primaryHandle_AuthSession,
@@ -203,13 +221,13 @@ test_esys_create_session_auth(ESYS_CONTEXT * esys_context)
 
     r = Esys_TR_SetAuth(esys_context, primaryHandle_AuthSession, &authValuePrimary);
     goto_if_error(r, "Error: TR_SetAuth", error);
-#else
+#elif defined(TEST_BOUND_SESSION) || !defined(TEST_NULL_BIND_NO_TPM_KEY)
     primaryHandle_AuthSession = primaryHandle;
 #endif /* TEST_ECC */
 
 #if TEST_XOR_OBFUSCATION
     TPMT_SYM_DEF symmetric = {.algorithm = TPM2_ALG_XOR,
-                              .keyBits = { .exclusiveOr = TPM2_ALG_SHA1 },
+                              .keyBits = { .exclusiveOr = TPM2_ALG_SHA256 },
                               .mode = {.aes = TPM2_ALG_CFB}};
 #elif TEST_AES_ENCRYPTION
     TPMT_SYM_DEF symmetric = {.algorithm = TPM2_ALG_AES,
@@ -228,15 +246,48 @@ test_esys_create_session_auth(ESYS_CONTEXT * esys_context)
     TPMI_ALG_HASH authHash = TPM2_ALG_SHA256;
 
     r = Esys_StartAuthSession(esys_context,
-                              primaryHandle_AuthSession,
-#if TEST_BOUND_SESSION
-                              primaryHandle_AuthSession,
-#else
-                              ESYS_TR_NONE,
-#endif
+                              ESYS_TR_NONE, ESYS_TR_NONE,
                               ESYS_TR_NONE, ESYS_TR_NONE, ESYS_TR_NONE,
                               NULL,
+                              TPM2_SE_HMAC, &symmetric, TPM2_ALG_SHA256,
+                              &outerSession);
+    goto_if_error(r, "Error during Esys_StartAuthSession", error);
+
+    r = Esys_TRSess_SetAttributes(esys_context, outerSession, TPMA_SESSION_AUDIT,
+                                  TPMA_SESSION_AUDIT);
+    goto_if_error(r, "Error Esys_TRSess_SetAttributes", error);
+
+#if defined(TEST_BOUND_SESSION)
+    r = Esys_StartAuthSession(esys_context,
+                              primaryHandle_AuthSession,
+                              primaryHandle_AuthSession,
+                              outerSession, ESYS_TR_NONE, ESYS_TR_NONE,
+                              NULL,
                               sessionType, &symmetric, authHash, &session);
+#elif defined(TEST_NULL_BIND_TPMKEY)
+     r = Esys_StartAuthSession(esys_context,
+                              primaryHandle_AuthSession,
+                              ESYS_TR_RH_NULL,
+                              outerSession, ESYS_TR_NONE, ESYS_TR_NONE,
+                              NULL,
+                              sessionType, &symmetric, authHash, &session);
+#elif defined(TEST_NULL_BIND_NO_TPM_KEY)
+     r = Esys_StartAuthSession(esys_context,
+                               ESYS_TR_NONE,
+                               ESYS_TR_RH_NULL,
+                               outerSession, ESYS_TR_NONE, ESYS_TR_NONE,
+                               NULL,
+                               sessionType, &symmetric, authHash, &session);
+#else
+     r = Esys_StartAuthSession(esys_context,
+                               primaryHandle_AuthSession,
+                               ESYS_TR_NONE,
+                               outerSession, ESYS_TR_NONE, ESYS_TR_NONE,
+                               NULL,
+                               sessionType, &symmetric, authHash, &session);
+#endif
+
+     Esys_FlushContext(esys_context, outerSession);
     goto_if_error(r, "Error during Esys_StartAuthSession", error);
 
 #ifdef TEST_ECC
@@ -358,12 +409,6 @@ test_esys_create_session_auth(ESYS_CONTEXT * esys_context)
         .count = 0,
     };
 
-    TPM2B_PUBLIC *outPublic2;
-    TPM2B_PRIVATE *outPrivate2;
-    TPM2B_CREATION_DATA *creationData2;
-    TPM2B_DIGEST *creationHash2;
-    TPMT_TK_CREATION *creationTicket2;
-
     r = Esys_Create(esys_context,
                     primaryHandle,
                     session, ESYS_TR_NONE, ESYS_TR_NONE,
@@ -390,6 +435,12 @@ test_esys_create_session_auth(ESYS_CONTEXT * esys_context)
     r = Esys_TR_SetAuth(esys_context, loadedKeyHandle, &authKey2);
     goto_if_error(r, "Error esys TR_SetAuth ", error);
 
+    Esys_Free(outPublic2);
+    Esys_Free(outPrivate2);
+    Esys_Free(creationData2);
+    Esys_Free(creationHash2);
+    Esys_Free(creationTicket2);
+
     r = Esys_Create(esys_context,
                     loadedKeyHandle,
                     session, ESYS_TR_NONE, ESYS_TR_NONE,
@@ -411,6 +462,23 @@ test_esys_create_session_auth(ESYS_CONTEXT * esys_context)
     r = Esys_FlushContext(esys_context, session);
     goto_if_error(r, "Flushing context", error);
 
+    Esys_Free(outPublic);
+    Esys_Free(creationData);
+    Esys_Free(creationHash);
+    Esys_Free(creationTicket);
+
+#ifdef TEST_ECC
+    Esys_Free(outPublicEcc);
+    Esys_Free(creationDataEcc);
+    Esys_Free(creationHashEcc);
+    Esys_Free(creationTicketEcc);
+#endif
+
+    Esys_Free(outPublic2);
+    Esys_Free(outPrivate2);
+    Esys_Free(creationData2);
+    Esys_Free(creationHash2);
+    Esys_Free(creationTicket2);
     return EXIT_SUCCESS;
 
  error:
@@ -441,11 +509,27 @@ test_esys_create_session_auth(ESYS_CONTEXT * esys_context)
     }
 #endif
 
+    Esys_Free(outPublic);
+    Esys_Free(creationData);
+    Esys_Free(creationHash);
+    Esys_Free(creationTicket);
 
+#ifdef TEST_ECC
+    Esys_Free(outPublicEcc);
+    Esys_Free(creationDataEcc);
+    Esys_Free(creationHashEcc);
+    Esys_Free(creationTicketEcc);
+#endif
+
+    Esys_Free(outPublic2);
+    Esys_Free(outPrivate2);
+    Esys_Free(creationData2);
+    Esys_Free(creationHash2);
+    Esys_Free(creationTicket2);
     return EXIT_FAILURE;
 }
 
 int
-test_invoke_esapi(ESYS_CONTEXT * esys_context) {
+test_invoke_esys(ESYS_CONTEXT * esys_context) {
     return test_esys_create_session_auth(esys_context);
 }
