@@ -75,7 +75,7 @@ test_fapi_key_create_policy_pcr_sign(FAPI_CONTEXT *context)
         "  \"description\":\"Description pol_16_0\"," \
         "  \"policyDigests\":[" \
         "    {" \
-        "      \"hashAlg\":\"SHA256\"," \
+        "      \"hashAlg\":\"sha256\"," \
         "      \"digest\":\"bff2d58e9813f97cefc14f72ad8133bc7092d652b7c877959254af140c841f36\"" \
         "    }" \
         "  ]," \
@@ -84,31 +84,64 @@ test_fapi_key_create_policy_pcr_sign(FAPI_CONTEXT *context)
         "      \"type\":\"POLICYPCR\"," \
         "      \"policyDigests\":[" \
         "        {" \
-        "          \"hashAlg\":\"SHA256\"," \
+        "          \"hashAlg\":\"sha256\"," \
         "          \"digest\":\"bff2d58e9813f97cefc14f72ad8133bc7092d652b7c877959254af140c841f36\"" \
         "        }" \
         "      ]," \
         "      \"pcrs\":[" \
         "        {" \
         "          \"pcr\":16," \
+        "          \"hashAlg\":\"sha256\"," \
+        "          \"digest\":\"0000000000000000000000000000000000000000000000000000000000000000\"" \
+        "        }" \
+        "      ]" \
+        "    }" \
+        "  ]" \
+        "}";
+
+    const char *policy_sha384_check =
+        "{" \
+        "  \"description\":\"Description pol_16_0\"," \
+        "  \"policyDigests\":[" \
+        "    {" \
+        "      \"hashAlg\":\"SHA384\"," \
+        "      \"digest\":\"c1923346b6d44a154b58b57b4327ee70c29ac536f9209d94880de6834f370587846a2834e3e88af61efd8679fcccedd5\"" \
+        "    }" \
+        "  ]," \
+        "  \"policy\":[" \
+        "    {" \
+        "      \"type\":\"POLICYPCR\"," \
+        "      \"policyDigests\":[" \
+        "        {" \
+        "          \"hashAlg\":\"SHA384\"," \
+        "          \"digest\":\"c1923346b6d44a154b58b57b4327ee70c29ac536f9209d94880de6834f370587846a2834e3e88af61efd8679fcccedd5\"" \
+        "        }" \
+        "      ]," \
+        "      \"pcrs\":[" \
+        "        {" \
+        "          \"pcr\":16," \
         "          \"hashAlg\":\"SHA256\"," \
         "          \"digest\":\"0000000000000000000000000000000000000000000000000000000000000000\"" \
         "        }" \
         "      ]" \
         "    }" \
         "  ]" \
-        "}";
+        "}" ;
 
     const char *policy_sha256_export_check =
         "{" \
         "  \"description\":\"Description pol_16_0\"," \
         "  \"policyDigests\":[" \
         "    {" \
-        "      \"hashAlg\":\"SHA256\"," \
+        "      \"hashAlg\":\"sha256\"," \
         "      \"digest\":\"bff2d58e9813f97cefc14f72ad8133bc7092d652b7c877959254af140c841f36\"" \
         "    }," \
+        "    { " \
+        "         \"hashAlg\":\"sha384\"," \
+        "         \"digest\":\"c1923346b6d44a154b58b57b4327ee70c29ac536f9209d94880de6834f370587846a2834e3e88af61efd8679fcccedd5\"" \
+        "    }," \
         "    {" \
-        "      \"hashAlg\":\"SHA1\"," \
+        "      \"hashAlg\":\"sha1\"," \
         "      \"digest\":\"eab0d71ae6088009cbd0b50729fde69eb453649c\"" \
         "    }" \
         "  ]," \
@@ -117,6 +150,55 @@ test_fapi_key_create_policy_pcr_sign(FAPI_CONTEXT *context)
         "      \"type\":\"POLICYPCR\"," \
         "      \"policyDigests\":[" \
         "        {" \
+        "          \"hashAlg\":\"sha256\"," \
+        "          \"digest\":\"bff2d58e9813f97cefc14f72ad8133bc7092d652b7c877959254af140c841f36\"" \
+        "        }," \
+        "        { " \
+        "          \"hashAlg\":\"sha384\"," \
+        "          \"digest\":\"c1923346b6d44a154b58b57b4327ee70c29ac536f9209d94880de6834f370587846a2834e3e88af61efd8679fcccedd5\"" \
+        "        }," \
+        "        {" \
+        "          \"hashAlg\":\"sha1\"," \
+        "          \"digest\":\"eab0d71ae6088009cbd0b50729fde69eb453649c\"" \
+        "        }" \
+        "      ]," \
+        "      \"pcrs\":[" \
+        "        {" \
+        "          \"pcr\":16," \
+        "          \"hashAlg\":\"sha256\"," \
+        "          \"digest\":\"0000000000000000000000000000000000000000000000000000000000000000\"" \
+        "        }" \
+        "      ]" \
+        "    }" \
+        "  ]" \
+        "}";
+
+       const char *policy_sha384_export_check =
+        "{" \
+        "  \"description\":\"Description pol_16_0\"," \
+        "  \"policyDigests\":[" \
+        "    { " \
+        "         \"hashAlg\":\"SHA384\"," \
+        "         \"digest\":\"c1923346b6d44a154b58b57b4327ee70c29ac536f9209d94880de6834f370587846a2834e3e88af61efd8679fcccedd5\"" \
+        "     }," \
+        "    {" \
+        "      \"hashAlg\":\"SHA256\"," \
+        "      \"digest\":\"bff2d58e9813f97cefc14f72ad8133bc7092d652b7c877959254af140c841f36\"" \
+        "    }," \
+        "    {" \
+        "      \"hashAlg\":\"SHA1\"," \
+        "      \"digest\":\"eab0d71ae6088009cbd0b50729fde69eb453649c\"" \
+        "    }"                                                        \
+        "  ]," \
+        "  \"policy\":[" \
+        "    {" \
+        "      \"type\":\"POLICYPCR\"," \
+        "      \"policyDigests\":[" \
+        "    { "                           \
+        "         \"hashAlg\":\"SHA384\"," \
+        "         \"digest\":\"c1923346b6d44a154b58b57b4327ee70c29ac536f9209d94880de6834f370587846a2834e3e88af61efd8679fcccedd5\"" \
+        "     }," \
+        "        {" \
         "          \"hashAlg\":\"SHA256\"," \
         "          \"digest\":\"bff2d58e9813f97cefc14f72ad8133bc7092d652b7c877959254af140c841f36\"" \
         "        }," \
@@ -136,6 +218,7 @@ test_fapi_key_create_policy_pcr_sign(FAPI_CONTEXT *context)
         "  ]" \
         "}";
 
+
     r = Fapi_Provision(context, NULL, NULL, NULL);
     goto_if_error(r, "Error Fapi_Provision", error);
 
@@ -198,9 +281,13 @@ test_fapi_key_create_policy_pcr_sign(FAPI_CONTEXT *context)
     ASSERT(policy != NULL);
     LOG_INFO("\nTEST_JSON\nPolicy_sha256:\n%s\nEND_JSON", policy);
 
-    CHECK_JSON(policy, policy_sha256_check, error);
+    if (strcmp(FAPI_PROFILE, "P_ECC384") == 0) {
+        CHECK_JSON(policy, policy_sha384_check, error);
+    } else {
+        CHECK_JSON(policy, policy_sha256_check, error);
+    }
+
     ASSERT(strlen(policy) > ASSERT_SIZE);
-    fprintf(stderr, "\nPolicy from key:\n%s\n", policy);
 
     SAFE_FREE(policy);
 
@@ -209,8 +296,11 @@ test_fapi_key_create_policy_pcr_sign(FAPI_CONTEXT *context)
     goto_if_error(r, "Error Fapi_ExportPolicy", error);
     ASSERT(policy != NULL);
     LOG_INFO("\nTEST_JSON\nPolicy export1:\n%s\nEND_JSON", policy);
-    CHECK_JSON(policy, policy_sha256_export_check, error);
-    fprintf(stderr, "\nPolicy from policy file:\n%s\n", policy);
+    if (strcmp(FAPI_PROFILE, "P_ECC384") == 0) {
+        CHECK_JSON(policy, policy_sha384_export_check, error)
+    } else {
+        CHECK_JSON(policy, policy_sha256_export_check, error)
+    }
 
     /* Run test with policy which should fail. */
     r = Fapi_Delete(context, "/HS/SRK/mySignKey");
@@ -336,7 +426,12 @@ test_fapi_key_create_policy_pcr_sign(FAPI_CONTEXT *context)
     r = Fapi_ExportPolicy(context, "HS/SRK/mySignKey", &policy);
     goto_if_error(r, "Error Fapi_ExportPolicy", error);
     ASSERT(policy != NULL);
-    CHECK_JSON(policy, policy_sha256_check, error);
+
+    if (strcmp(FAPI_PROFILE, "P_ECC384") == 0) {
+        CHECK_JSON(policy, policy_sha384_check, error);
+    } else {
+        CHECK_JSON(policy, policy_sha256_check, error);
+    }
     fprintf(stderr, "\nPolicy from key:\n%s\n", policy);
 
     jso = json_tokener_parse(policy);
