@@ -20,14 +20,12 @@
 #include "tss2_fapi.h"
 
 #include "test-fapi.h"
+#include "ifapi_eventlog.h"
 #define LOGMODULE test
 #include "util/log.h"
 #include "util/aux_util.h"
 
 #define EVENT_SIZE 10
-#ifndef FAPI_PROFILE
-#define FAPI_PROFILE "P_ECC"
-#endif /* FAPI_PROFILE */
 
 /** Test the FAPI functions for quote commands.
  *
@@ -149,64 +147,28 @@ test_fapi_quote(FAPI_CONTEXT *context)
         {
          "["
          "  {"
-         "    \"recnum\":1,"
+         "    \"recnum\":0,"
          "    \"pcr\":16,"
          "    \"digests\":["
          "      {"
-         "        \"hashAlg\":\"SHA1\","
+         "        \"hashAlg\":\"sha1\","
          "        \"digest\":\"494179714a6cd627239dfededf2de9ef994caf03\""
          "      },"
          "      {"
-         "        \"hashAlg\":\"SHA256\","
+         "        \"hashAlg\":\"sha256\","
          "        \"digest\":\"1f825aa2f0020ef7cf91dfa30da4668d791c5d4824fc8e41354b89ec05795ab3\""
          "      },"
          "      {"
-         "        \"hashAlg\":\"SHA384\","
+         "        \"hashAlg\":\"sha384\","
          "        \"digest\":\"182e95266adff49059e706c61483478fe0688150c8d08b95fab5cfde961f12d903aaf44104af4ce72ba6a4bf20302b2e\""
          "      },"
          "      {"
-         "        \"hashAlg\":\"SHA512\","
-         "        \"digest\":\"0f89ee1fcb7b0a4f7809d1267a029719004c5a5e5ec323a7c3523a20974f9a3f202f56fadba4cd9e8d654ab9f2e96dc5c795ea176fa20ede8d854c342f903533\""
-         "      },"
-         "      {"
-         "        \"hashAlg\":\"SM3_256\","
-         "        \"digest\":\"24c898bdb4d258f9bebb2e820d4ed478a7c013b37bd9e5006515730c18a70416\""
-         "      },"
-
-         "    ],"
-         "    \"type\":\"tss2\","
-         "    \"sub_event\":{"
-         "      \"data\":\"00010203040506070809\","
-         "      \"event\":{"
-         "        \"test\":\"myfile\""
-         "      }"
-         "    }"
-         "  }"
-         "]",
-         "["
-         "  {"
-         "    \"recnum\":1,"
-         "    \"pcr\":16,"
-         "    \"digests\":["
-         "      {"
-         "        \"hashAlg\":\"SHA1\","
-         "        \"digest\":\"494179714a6cd627239dfededf2de9ef994caf03\""
-         "      },"
-         "      {"
-         "        \"hashAlg\":\"SHA256\","
-         "        \"digest\":\"1f825aa2f0020ef7cf91dfa30da4668d791c5d4824fc8e41354b89ec05795ab3\""
-         "      },"
-         "      {"
-         "        \"hashAlg\":\"SHA384\","
-         "        \"digest\":\"182e95266adff49059e706c61483478fe0688150c8d08b95fab5cfde961f12d903aaf44104af4ce72ba6a4bf20302b2e\""
-         "      },"
-         "      {"
-         "        \"hashAlg\":\"SHA512\","
+         "        \"hashAlg\":\"sha512\","
          "        \"digest\":\"0f89ee1fcb7b0a4f7809d1267a029719004c5a5e5ec323a7c3523a20974f9a3f202f56fadba4cd9e8d654ab9f2e96dc5c795ea176fa20ede8d854c342f903533\""
          "      }"
          "    ],"
-         "    \"type\":\"tss2\","
-         "    \"sub_event\":{"
+         "    \"" CONTENT_TYPE "\":\"tss2\","
+         "    \"" CONTENT "\":{"
          "      \"data\":\"00010203040506070809\","
          "      \"event\":{"
          "        \"test\":\"myfile\""
@@ -216,24 +178,24 @@ test_fapi_quote(FAPI_CONTEXT *context)
          "]",
          "["
          "  {"
-         "    \"recnum\":1,"
+         "    \"recnum\":0,"
          "    \"pcr\":16,"
          "    \"digests\":["
          "      {"
-         "        \"hashAlg\":\"SHA1\","
+         "        \"hashAlg\":\"sha1\","
          "        \"digest\":\"494179714a6cd627239dfededf2de9ef994caf03\""
          "      },"
          "      {"
-         "        \"hashAlg\":\"SHA256\","
+         "        \"hashAlg\":\"sha256\","
          "        \"digest\":\"1f825aa2f0020ef7cf91dfa30da4668d791c5d4824fc8e41354b89ec05795ab3\""
          "      },"
          "      {"
-         "        \"hashAlg\":\"SHA384\","
+         "        \"hashAlg\":\"sha384\","
          "        \"digest\":\"182e95266adff49059e706c61483478fe0688150c8d08b95fab5cfde961f12d903aaf44104af4ce72ba6a4bf20302b2e\""
          "      }"
          "    ],"
-         "    \"type\":\"tss2\","
-         "    \"sub_event\":{"
+         "    \"" CONTENT_TYPE "\":\"tss2\","
+         "    \"" CONTENT "\":{"
          "      \"data\":\"00010203040506070809\","
          "      \"event\":{"
          "        \"test\":\"myfile\""
@@ -243,20 +205,20 @@ test_fapi_quote(FAPI_CONTEXT *context)
          "]",
          "["
          "  {"
-         "    \"recnum\":1,"
+         "    \"recnum\":0,"
          "    \"pcr\":16,"
          "    \"digests\":["
          "      {"
-         "        \"hashAlg\":\"SHA1\","
+         "        \"hashAlg\":\"sha1\","
          "        \"digest\":\"494179714a6cd627239dfededf2de9ef994caf03\""
          "      },"
          "      {"
-         "        \"hashAlg\":\"SHA256\","
+         "        \"hashAlg\":\"sha256\","
          "        \"digest\":\"1f825aa2f0020ef7cf91dfa30da4668d791c5d4824fc8e41354b89ec05795ab3\""
          "      }"
          "    ],"
-         "    \"type\":\"tss2\","
-         "    \"sub_event\":{"
+         "    \"" CONTENT_TYPE "\":\"tss2\","
+         "    \"" CONTENT "\":{"
          "      \"data\":\"00010203040506070809\","
          "      \"event\":{"
          "        \"test\":\"myfile\""
@@ -266,16 +228,16 @@ test_fapi_quote(FAPI_CONTEXT *context)
          "]",
          "["
          "  {"
-         "    \"recnum\":1,"
+         "    \"recnum\":0,"
          "    \"pcr\":16,"
          "    \"digests\":["
          "      {"
-         "        \"hashAlg\":\"SHA1\","
+         "        \"hashAlg\":\"sha1\","
          "        \"digest\":\"494179714a6cd627239dfededf2de9ef994caf03\""
          "      }"
          "    ],"
-         "    \"type\":\"tss2\","
-         "    \"sub_event\":{"
+         "    \"" CONTENT_TYPE "\":\"tss2\","
+         "    \"" CONTENT "\":{"
          "      \"data\":\"00010203040506070809\","
          "      \"event\":{"
          "        \"test\":\"myfile\""
@@ -284,6 +246,7 @@ test_fapi_quote(FAPI_CONTEXT *context)
          "  }"
          "]"
         };
+
     CHECK_JSON_LIST(log_check_list, log, error);
 
     r = Fapi_VerifyQuote(context, "HS/mySignKey",
