@@ -17,10 +17,15 @@
 
 #include "tss2_sys.h"
 #include "tss2_tcti_device.h"
+#ifdef TCTI_MSSIM
 #include "tss2_tcti_mssim.h"
+#endif /* TCTI_MSSIM */
+#ifdef TCTI_SWTPM
+#include "tss2_tcti_swtpm.h"
+#endif /* TCTI_SWTPM */
 
 #include "../integration/context-util.h"
-#include "../integration/sapi-util.h"
+#include "../integration/sys-util.h"
 #include "../integration/session-util.h"
 #include "util/tss2_endian.h"
 #include "sysapi_util.h"
@@ -96,15 +101,20 @@ static void ErrorHandler(UINT32 rval, char *errorString, int errorStringSize)
     snprintf(errorString, errorStringSize, "%s Error: 0x%x\n", levelString, rval);
 }
 
-static void Cleanup()
+#define EXIT_SKIP 77
+
+static void Cleanup_exit(int rc)
 {
     if (resMgrTctiContext != NULL) {
-        tcti_platform_command(resMgrTctiContext, MS_SIM_POWER_OFF);
         tcti_teardown(resMgrTctiContext);
         resMgrTctiContext = NULL;
     }
+    exit(rc == EXIT_SKIP ? EXIT_SKIP : 1);
+}
 
-    exit(1);
+static void Cleanup()
+{
+    Cleanup_exit(1);
 }
 
 static void InitSysContextFailure()
@@ -120,7 +130,7 @@ static void InitSysContextFailure()
       ErrorHandler((rval), error_string, ERROR_STR_LEN); \
       LOG_INFO("passing case: \tFAILED!  %s (%s@%u)",  \
                error_string, __FUNCTION__, __LINE__ ); \
-      Cleanup(); \
+      Cleanup_exit(rval); \
     } else {     \
       LOG_INFO("passing case: \tPASSED! (%s@%u)", \
                __FUNCTION__, __LINE__); \
@@ -133,7 +143,7 @@ static void InitSysContextFailure()
       ErrorHandler((rval), error_string, ERROR_STR_LEN); \
       LOG_INFO("\tfailing case: FAILED! %s  Ret code s/b: 0x%x, but was: 0x%x (%s@%u)", \
                error_string, (expected_rval), (rval), __FUNCTION__, __LINE__ ); \
-      Cleanup(); \
+      Cleanup_exit(rval); \
     } else { \
       LOG_INFO("\tfailing case: PASSED! (%s@%u)", \
            __FUNCTION__, __LINE__); \
@@ -144,11 +154,27 @@ static TSS2_RC TpmReset()
 {
     TSS2_RC rval = TSS2_RC_SUCCESS;
 
+#ifdef TCTI_SWTPM
+    rval = Tss2_Tcti_Swtpm_Reset(resMgrTctiContext);
+
+    /* If TCTI is not swtpm, bad context is returned. */
+    if (rval != TSS2_TCTI_RC_BAD_CONTEXT) {
+        return rval;
+    } else {
+        LOG_WARNING("TPM Reset failed: wrong TCTI type retrying with mssim...");
+    }
+#endif /* TCTI_SWTPM */
+
+#ifdef TCTI_MSSIM
     rval = (TSS2_RC)tcti_platform_command( resMgrTctiContext, MS_SIM_POWER_OFF );
-    if( rval == TSS2_RC_SUCCESS )
-    {
+    if (rval == TSS2_RC_SUCCESS) {
         rval = (TSS2_RC)tcti_platform_command( resMgrTctiContext, MS_SIM_POWER_ON );
     }
+#endif /* TCTI_MSSIM */
+    if (rval == TSS2_TCTI_RC_BAD_CONTEXT) {
+        rval = EXIT_SKIP;
+    }
+
     return rval;
 }
 
@@ -158,7 +184,7 @@ static void TestDictionaryAttackLockReset()
     TSS2L_SYS_AUTH_RESPONSE sessionsDataOut;
 
     TSS2L_SYS_AUTH_COMMAND sessionsData = { .count = 1, .auths = {{
-        .sessionHandle = TPM2_RS_PW,
+        .sessionHandle = TPM2_RH_PW,
         .sessionAttributes = 0,
         .nonce={.size=0},
         .hmac={.size=0}}}};
@@ -193,10 +219,8 @@ static void TestTpmStartup()
 
 
     /* Cycle power using simulator interface. */
-    rval = tcti_platform_command( resMgrTctiContext, MS_SIM_POWER_OFF );
-    CheckPassed( rval );
-    rval = tcti_platform_command( resMgrTctiContext, MS_SIM_POWER_ON );
-    CheckPassed( rval );
+    rval = TpmReset();
+    CheckPassed(rval);
 
 
     /*
@@ -210,10 +234,8 @@ static void TestTpmStartup()
     CheckPassed( rval );
 
     /* Cycle power using simulator interface. */
-    rval = tcti_platform_command( resMgrTctiContext, MS_SIM_POWER_OFF );
-    CheckPassed( rval );
-    rval = tcti_platform_command( resMgrTctiContext, MS_SIM_POWER_ON );
-    CheckPassed( rval );
+    rval = TpmReset();
+    CheckPassed(rval);
 
 
     /*
@@ -289,7 +311,7 @@ static void TestTpmClear()
     TSS2L_SYS_AUTH_RESPONSE sessionsDataOut;
 
     TSS2L_SYS_AUTH_COMMAND sessionsDataIn = { .count = 1, .auths = {{
-        .sessionHandle = TPM2_RS_PW,
+        .sessionHandle = TPM2_RH_PW,
         .sessionAttributes = 0,
         .nonce=nonce,
         .hmac=hmac}}};
@@ -411,6 +433,7 @@ static void TestStartAuthSession()
 
         end_auth_session(sessions[i]);
     }
+    end_auth_session(sessions[0]);
 
     for( i = 0; i < ( sizeof(sessions) / sizeof (SESSION *) ); i++ )
     {
@@ -429,7 +452,7 @@ static void TestChangeEps()
     UINT32 rval;
     TSS2L_SYS_AUTH_RESPONSE sessionsDataOut;
     TSS2L_SYS_AUTH_COMMAND sessionsData = { .count = 1, .auths = {{
-        .sessionHandle = TPM2_RS_PW,
+        .sessionHandle = TPM2_RH_PW,
         .sessionAttributes = 0,
         .nonce = {.size = 0},
         .hmac = {.size = 0}}}};
@@ -447,7 +470,7 @@ static void TestChangePps()
     TSS2L_SYS_AUTH_RESPONSE sessionsDataOut;
 
     TSS2L_SYS_AUTH_COMMAND sessionsData = { .count = 1, .auths = {{
-        .sessionHandle = TPM2_RS_PW,
+        .sessionHandle = TPM2_RH_PW,
         .sessionAttributes = 0,
         .nonce = {.size = 0},
         .hmac = {.size = 0}}}};
@@ -465,7 +488,7 @@ static void TestHierarchyChangeAuth()
     int i;
 
     TSS2L_SYS_AUTH_COMMAND sessionsData = { .count = 1, .auths = {{
-        .sessionHandle = TPM2_RS_PW,
+        .sessionHandle = TPM2_RH_PW,
         .sessionAttributes = 0,
         .nonce = {.size = 0},
         .hmac = {.size = 0}}}};
@@ -538,7 +561,7 @@ static void TestPcrExtend()
     TSS2_TCTI_CONTEXT *tctiContext;
 
     TSS2L_SYS_AUTH_COMMAND sessionsData = { .count = 1, .auths = {{
-        .sessionHandle = TPM2_RS_PW,
+        .sessionHandle = TPM2_RH_PW,
         .sessionAttributes = 0,
         .nonce = {.size = 0},
         .hmac = {.size = 0}}}};
@@ -654,7 +677,7 @@ static void TestNV()
     TPM2B_NAME nvName;
 
     TSS2L_SYS_AUTH_COMMAND sessionsData = { .count = 1, .auths = {{
-        .sessionHandle = TPM2_RS_PW,
+        .sessionHandle = TPM2_RH_PW,
         .sessionAttributes = 0,
         .nonce = {.size = 0},
         .hmac = {.size = 0}}}};
@@ -792,7 +815,7 @@ static void TestHierarchyControl()
     TPM2B_MAX_NV_BUFFER nvData;
 
     TSS2L_SYS_AUTH_COMMAND sessionsData = { .count = 1, .auths = {{
-        .sessionHandle = TPM2_RS_PW,
+        .sessionHandle = TPM2_RH_PW,
         .sessionAttributes = 0,
         .nonce = {.size = 0},
         .hmac = {.size = 0}}}};
@@ -824,7 +847,7 @@ static void TestHierarchyControl()
     rval = Tss2_Sys_NV_DefineSpace( sysContext, TPM2_RH_PLATFORM, &sessionsData, &nvAuth, &publicInfo, 0 );
     CheckPassed( rval );
 
-    /* Test SAPI for case where nvPublic.size != 0 */
+    /* Test SYS for case where nvPublic.size != 0 */
     nvPublic.size = 0xff;
     INIT_SIMPLE_TPM2B_SIZE( nvName );
     rval = Tss2_Sys_NV_ReadPublic( sysContext, TPM20_INDEX_TEST1, 0, &nvPublic, &nvName, 0 );
@@ -862,36 +885,6 @@ static void TestHierarchyControl()
     CheckPassed( rval );
 }
 
-static TSS2_RC DefineNvIndex( TPMI_RH_PROVISION authHandle, TPMI_SH_AUTH_SESSION sessionAuthHandle, TPM2B_AUTH *auth, TPM2B_DIGEST *authPolicy,
-    TPMI_RH_NV_INDEX nvIndex, TPMI_ALG_HASH nameAlg, TPMA_NV attributes, UINT16 size  )
-{
-    TSS2_RC rval = TPM2_RC_SUCCESS;
-    TPM2B_NV_PUBLIC publicInfo;
-
-    /* Command and response session data structures. */
-    TSS2L_SYS_AUTH_RESPONSE sessionsDataOut;
-    TSS2L_SYS_AUTH_COMMAND sessionsData = { .count = 1, .auths = {{
-        .sessionHandle = TPM2_RS_PW,
-        .sessionAttributes = 0,
-        .nonce = {.size = 0},
-        .hmac = {.size = 0}}}};
-
-    attributes |= TPMA_NV_ORDERLY;
-
-    /* Init public info structure. */
-    publicInfo.size = 0;
-    publicInfo.nvPublic.attributes = attributes;
-    CopySizedByteBuffer((TPM2B *)&publicInfo.nvPublic.authPolicy, (TPM2B *)authPolicy);
-    publicInfo.nvPublic.dataSize = size;
-    publicInfo.nvPublic.nvIndex = nvIndex;
-    publicInfo.nvPublic.nameAlg = nameAlg;
-
-    /* Create the index */
-    rval = Tss2_Sys_NV_DefineSpace( sysContext, authHandle, &sessionsData, auth, &publicInfo, &sessionsDataOut );
-
-    return rval;
-}
-
 typedef struct {
     char name[50];
     TSS2_RC (*buildPolicyFn )( TSS2_SYS_CONTEXT *sysContext, SESSION *trialPolicySession, TPM2B_DIGEST *policyDigest );
@@ -988,7 +981,7 @@ static TSS2_RC CreateNVIndex( TSS2_SYS_CONTEXT *sysContext, SESSION **policySess
     nvAttributes |= TPMA_NV_POLICYWRITE;
     nvAttributes |= TPMA_NV_PLATFORMCREATE;
 
-    rval = DefineNvIndex( TPM2_RH_PLATFORM, TPM2_RS_PW, &nvAuth, policyDigest,
+    rval = DefineNvIndex( sysContext, TPM2_RH_PLATFORM, &nvAuth, policyDigest,
             TPM20_INDEX_PASSWORD_TEST, TPM2_ALG_SHA256, nvAttributes, 32  );
     CheckPassed( rval );
 
@@ -1043,7 +1036,7 @@ static TSS2_RC TestLocality( TSS2_SYS_CONTEXT *sysContext, SESSION *policySessio
     CheckFailed( rval, TPM2_RC_POLICY_FAIL + TPM2_RC_S + TPM2_RC_1 );
 
     /* Delete NV index */
-    sessionsData.auths[0].sessionHandle = TPM2_RS_PW;
+    sessionsData.auths[0].sessionHandle = TPM2_RH_PW;
     sessionsData.auths[0].nonce.size = 0;
     sessionsData.auths[0].nonce.buffer[0] = 0xa5;
     sessionsData.auths[0].hmac.size = 0;
@@ -1094,7 +1087,7 @@ static TSS2_RC CreateDataBlob( TSS2_SYS_CONTEXT *sysContext, SESSION **policySes
     TPM2B_PRIVATE outPrivate;
 
     TSS2L_SYS_AUTH_COMMAND cmdAuthArray = { .count = 1, .auths = {{
-        .sessionHandle = TPM2_RS_PW,
+        .sessionHandle = TPM2_RH_PW,
         .sessionAttributes = 0,
         .nonce = {.size = 0},
         .hmac = {.size = 0}}}};
@@ -1132,7 +1125,7 @@ static TSS2_RC CreateDataBlob( TSS2_SYS_CONTEXT *sysContext, SESSION **policySes
             &creationTicket, &srkName, 0 );
     CheckPassed( rval );
 
-    cmdAuthArray.auths[0].sessionHandle = TPM2_RS_PW;
+    cmdAuthArray.auths[0].sessionHandle = TPM2_RH_PW;
 
     inSensitive.sensitive.userAuth.size = 0;
     blobAuth.size = sizeof( passwordPCRTestPassword );
@@ -1385,7 +1378,7 @@ static void TestHash()
     rval = Tss2_Sys_HashSequenceStart ( sysContext, 0, &auth, TPM2_ALG_SHA1, &sequenceHandle[0], 0 );
     CheckPassed( rval );
 
-    sessionsData.auths[0].sessionHandle = TPM2_RS_PW;
+    sessionsData.auths[0].sessionHandle = TPM2_RH_PW;
     sessionsData.auths[0].nonce.size = 0;
     sessionsData.auths[0].hmac = auth;
     sessionsData.auths[0].sessionAttributes = 0;
@@ -1422,7 +1415,7 @@ static void TestQuote()
     TPMT_SIGNATURE signature;
     TSS2L_SYS_AUTH_COMMAND sessionsData = { .count = 1,
         .auths = {{
-            .sessionHandle = TPM2_RS_PW,
+            .sessionHandle = TPM2_RH_PW,
             .sessionAttributes = 0,
             .nonce = { .size = 0 },
             .hmac = { .size = 0, .buffer={0x00} },
@@ -1484,7 +1477,7 @@ static void TestPcrAllocate()
     UINT32 sizeAvailable;
 
     TSS2L_SYS_AUTH_COMMAND sessionsData = { .count = 1, .auths= {{
-        .sessionHandle = TPM2_RS_PW,
+        .sessionHandle = TPM2_RH_PW,
         .sessionAttributes = 0,
         .nonce={.size=0},
         .hmac={.size=0}}}};
@@ -1530,7 +1523,7 @@ static void TestUnseal()
 
     TSS2L_SYS_AUTH_COMMAND sessionsData = { .count = 1,
         .auths= {{
-            .sessionHandle = TPM2_RS_PW,
+            .sessionHandle = TPM2_RH_PW,
             .sessionAttributes = 0,
             .nonce={.size=0},
             .hmac={.size=0, .buffer={0x00}}
@@ -1629,7 +1622,7 @@ static void CreatePasswordTestNV( TPMI_RH_NV_INDEX nvIndex, char * password )
 
 
     TSS2L_SYS_AUTH_COMMAND sessionsData = { .count = 1, .auths= {{
-        .sessionHandle = TPM2_RS_PW,
+        .sessionHandle = TPM2_RH_PW,
         .sessionAttributes = 0,
         .nonce={.size=0},
         .hmac={.size=0}}}};
@@ -1670,7 +1663,7 @@ static void PasswordTest()
 
     /* Authorization array for command (only has one auth structure). */
     TSS2L_SYS_AUTH_COMMAND sessionsData = { .count = 1, .auths= {{
-        .sessionHandle = TPM2_RS_PW,
+        .sessionHandle = TPM2_RH_PW,
         .sessionAttributes = 0,
         .nonce={.size=0},
         .hmac={.size=0}}}};
@@ -1699,11 +1692,13 @@ static void PasswordTest()
      * Attempt write with the correct password.
      * It should pass.
      */
-    rval = Tss2_Sys_NV_Write( sysContext,
-            TPM20_INDEX_PASSWORD_TEST,
-            TPM20_INDEX_PASSWORD_TEST,
-            &sessionsData, &nvWriteData, 0,
-            &sessionsDataOut );
+    do {
+        rval = Tss2_Sys_NV_Write( sysContext,
+                TPM20_INDEX_PASSWORD_TEST,
+                TPM20_INDEX_PASSWORD_TEST,
+                &sessionsData, &nvWriteData, 0,
+                &sessionsDataOut );
+    } while (rval == TPM2_RC_RETRY);
     /*
      * Check that the function passed as
      * expected.  Otherwise, exit.
@@ -1737,377 +1732,6 @@ static void PasswordTest()
     CheckPassed( rval );
 }
 
-static void SimpleHmacOrPolicyTest( bool hmacTest )
-{
-    UINT32 rval, sessionCmdRval;
-    TPM2B_AUTH  nvAuth;
-    SESSION *nvSession, *trialPolicySession;
-    TPMA_NV nvAttributes;
-    TPM2B_DIGEST authPolicy;
-    TPM2B_NAME nvName;
-    TPM2B_MAX_NV_BUFFER nvWriteData, nvReadData;
-    UINT8 dataToWrite[] = { 0x00, 0xff, 0x55, 0xaa };
-    char sharedSecret[] = "shared secret";
-    int i;
-    TPM2B_ENCRYPTED_SECRET encryptedSalt;
-    TPMT_SYM_DEF symmetric;
-    TPM2_SE tpmSe;
-    char *testString;
-    char testStringHmac[] = "HMAC";
-    char testStringPolicy[] = "POLICY";
-
-    /* Response authorization area. */
-    TSS2L_SYS_AUTH_RESPONSE nvRspAuths;
-
-    /* Command authorization area: one password session. */
-    TSS2L_SYS_AUTH_COMMAND nvCmdAuths = { .count = 1, .auths= {{
-        .sessionHandle = TPM2_RS_PW,
-        .sessionAttributes = 0,
-        .nonce={.size=0},
-        .hmac={.size=0}}}};
-
-    TSS2_SYS_CONTEXT *simpleTestContext;
-    TPM2B_NONCE nonceCaller;
-
-    nonceCaller.size = 0;
-
-    if( hmacTest )
-        testString = testStringHmac;
-    else
-        testString = testStringPolicy;
-
-    LOG_INFO("SIMPLE %s SESSION TEST:", testString );
-    /* If LOG_INFO is not compiled in, this variable is unused */
-    (void)(testString);
-
-    /* Create sysContext structure. */
-    simpleTestContext = sapi_init_from_tcti_ctx(resMgrTctiContext);
-    if (simpleTestContext == NULL)
-        InitSysContextFailure();
-
-    /* Setup the NV index's authorization value. */
-    nvAuth.size = strlen( sharedSecret );
-    for( i = 0; i < nvAuth.size; i++ )
-        nvAuth.buffer[i] = sharedSecret[i];
-
-    /*
-     * Create NV index.
-     */
-    if( hmacTest )
-    {
-        /* Setup the NV index's authorization value. */
-        nvAuth.size = strlen( sharedSecret );
-        for( i = 0; i < nvAuth.size; i++ )
-            nvAuth.buffer[i] = sharedSecret[i];
-
-        /*
-         * Set NV index's authorization policy
-         * to zero sized policy since we won't be
-         * using policy to authorize.
-         */
-
-        authPolicy.size = 0;
-    }
-    else
-    {
-        /*
-         * Zero sized encrypted salt, since the session
-         * is unsalted.
-         */
-
-        encryptedSalt.size = 0;
-
-        /* No symmetric algorithm. */
-        symmetric.algorithm = TPM2_ALG_NULL;
-
-        /*
-         * Create the NV index's authorization policy
-         * using a trial policy session.
-         */
-        rval = create_auth_session(&trialPolicySession,
-                TPM2_RH_NULL, 0, TPM2_RH_NULL, 0, &nonceCaller, &encryptedSalt,
-                TPM2_SE_TRIAL,
-                &symmetric, TPM2_ALG_SHA256, resMgrTctiContext );
-        CheckPassed( rval );
-
-        rval = Tss2_Sys_PolicyAuthValue( simpleTestContext,
-                trialPolicySession->sessionHandle, 0, 0 );
-        CheckPassed( rval );
-
-        /* Get policy digest. */
-        INIT_SIMPLE_TPM2B_SIZE( authPolicy );
-        rval = Tss2_Sys_PolicyGetDigest( simpleTestContext,
-                trialPolicySession->sessionHandle,
-                0, &authPolicy, 0 );
-        CheckPassed( rval );
-
-        /* End the trial session by flushing it. */
-        rval = Tss2_Sys_FlushContext( simpleTestContext,
-                trialPolicySession->sessionHandle );
-        CheckPassed( rval );
-
-        /*
-         * And remove the trial policy session from
-         * sessions table.
-         */
-        end_auth_session( trialPolicySession );
-    }
-
-    /*
-     * Now set the NV index's attributes:
-     * policyRead, authWrite, and platormCreate.
-     */
-    *(UINT32 *)( &nvAttributes ) = 0;
-    if( hmacTest )
-    {
-        nvAttributes |= TPMA_NV_AUTHREAD;
-        nvAttributes |= TPMA_NV_AUTHWRITE;
-    }
-    else
-    {
-        nvAttributes |= TPMA_NV_POLICYREAD;
-        nvAttributes |= TPMA_NV_POLICYWRITE;
-    }
-    nvAttributes |= TPMA_NV_PLATFORMCREATE;
-
-    /* Create the NV index. */
-    rval = DefineNvIndex( TPM2_RH_PLATFORM, TPM2_RS_PW,
-            &nvAuth, &authPolicy, TPM20_INDEX_PASSWORD_TEST,
-            TPM2_ALG_SHA256, nvAttributes, 32  );
-    CheckPassed( rval );
-
-    /*
-     * Add index and associated authorization value to
-     * entity table.  This helps when we need
-     * to calculate HMACs.
-     */
-    rval = AddEntity(TPM20_INDEX_PASSWORD_TEST, &nvAuth);
-    CheckPassed(rval);
-
-    /* Get the name of the NV index. */
-    rval = tpm_handle_to_name(
-            resMgrTctiContext,
-            TPM20_INDEX_PASSWORD_TEST,
-            &nvName);
-    CheckPassed( rval );
-
-
-    /*
-     * Start HMAC or real (non-trial) policy authorization session:
-     * it's an unbound and unsalted session, no symmetric
-     * encryption algorithm, and SHA256 is the session's
-     * hash algorithm.
-     */
-
-    /*
-     * Zero sized encrypted salt, since the session
-     * is unsalted.
-     */
-    encryptedSalt.size = 0;
-
-    /* No symmetric algorithm. */
-    symmetric.algorithm = TPM2_ALG_NULL;
-
-    /*
-     * Create the session, hmac or policy depending
-     * on hmacTest.
-     * Session state (session handle, nonces, etc.) gets
-     * saved into nvSession structure for later use.
-     */
-    if( hmacTest )
-        tpmSe = TPM2_SE_HMAC;
-    else
-        tpmSe = TPM2_SE_POLICY;
-
-    rval = create_auth_session(&nvSession, TPM2_RH_NULL,
-            0, TPM2_RH_NULL, 0, &nonceCaller, &encryptedSalt, tpmSe,
-            &symmetric, TPM2_ALG_SHA256, resMgrTctiContext );
-    CheckPassed( rval );
-
-    /*
-     * Get the name of the session and save it in
-     * the nvSession structure.
-     */
-    rval = tpm_handle_to_name(
-            resMgrTctiContext,
-            nvSession->sessionHandle,
-            &(nvSession->name) );
-    CheckPassed( rval );
-
-    /* Initialize NV write data. */
-    nvWriteData.size = sizeof( dataToWrite );
-    for( i = 0; i < nvWriteData.size; i++ )
-    {
-        nvWriteData.buffer[i] = dataToWrite[i];
-    }
-
-    /*
-     * Now setup for writing the NV index.
-     */
-    if( !hmacTest )
-    {
-        /* Send policy command. */
-        rval = Tss2_Sys_PolicyAuthValue( simpleTestContext,
-                nvSession->sessionHandle, 0, 0 );
-        CheckPassed( rval );
-    }
-
-    /* First call prepare in order to create cpBuffer. */
-    rval = Tss2_Sys_NV_Write_Prepare( simpleTestContext,
-            TPM20_INDEX_PASSWORD_TEST,
-            TPM20_INDEX_PASSWORD_TEST, &nvWriteData, 0 );
-    CheckPassed( rval );
-
-    /* Configure command authorization area, except for HMAC. */
-    nvCmdAuths.auths[0].sessionHandle = nvSession->sessionHandle;
-    nvCmdAuths.auths[0].nonce.size = 1;
-    nvCmdAuths.auths[0].nonce.buffer[0] = 0xa5;
-    nvCmdAuths.auths[0].sessionAttributes = 0;
-    nvCmdAuths.auths[0].sessionAttributes |= TPMA_SESSION_CONTINUESESSION;
-
-    /* Roll nonces for command */
-    roll_nonces(nvSession, &nvCmdAuths.auths[0].nonce );
-
-    /*
-     * Complete command authorization area, by computing
-     * HMAC and setting it in nvCmdAuths.
-     */
-    rval = compute_command_hmac(
-            simpleTestContext,
-            TPM20_INDEX_PASSWORD_TEST,
-            TPM20_INDEX_PASSWORD_TEST,
-            TPM2_RH_NULL,
-            &nvCmdAuths);
-    CheckPassed(rval);
-
-    /*
-     * Finally!!  Write the data to the NV index.
-     * If the command is successful, the command
-     * HMAC was correct.
-     */
-    sessionCmdRval = Tss2_Sys_NV_Write(simpleTestContext,
-            TPM20_INDEX_PASSWORD_TEST,
-            TPM20_INDEX_PASSWORD_TEST,
-            &nvCmdAuths, &nvWriteData, 0, &nvRspAuths);
-    CheckPassed(sessionCmdRval);
-
-    /* Roll nonces for response */
-    roll_nonces(nvSession, &nvRspAuths.auths[0].nonce );
-
-    if (sessionCmdRval == TPM2_RC_SUCCESS) {
-        /*
-         * If the command was successful, check the
-         * response HMAC to make sure that the
-         * response was received correctly.
-         */
-        rval = check_response_hmac(
-                simpleTestContext,
-                &nvCmdAuths,
-                TPM20_INDEX_PASSWORD_TEST,
-                TPM20_INDEX_PASSWORD_TEST,
-                TPM2_RH_NULL,
-                &nvRspAuths);
-        CheckPassed(rval);
-    }
-
-    if( !hmacTest )
-    {
-        /* Send policy command. */
-        rval = Tss2_Sys_PolicyAuthValue( simpleTestContext,
-                nvSession->sessionHandle, 0, 0 );
-        CheckPassed( rval );
-    }
-
-    /* First call prepare in order to create cpBuffer. */
-    rval = Tss2_Sys_NV_Read_Prepare( simpleTestContext,
-            TPM20_INDEX_PASSWORD_TEST,
-            TPM20_INDEX_PASSWORD_TEST,
-            sizeof( dataToWrite ), 0 );
-    CheckPassed( rval );
-
-    /* Roll nonces for command */
-    roll_nonces(nvSession, &nvCmdAuths.auths[0].nonce );
-
-    /* End the session after next command. */
-    nvCmdAuths.auths[0].sessionAttributes &= ~TPMA_SESSION_CONTINUESESSION;
-
-    /*
-     * Complete command authorization area, by computing
-     * HMAC and setting it in nvCmdAuths.
-     */
-    rval = compute_command_hmac(
-            simpleTestContext,
-            TPM20_INDEX_PASSWORD_TEST,
-            TPM20_INDEX_PASSWORD_TEST,
-            TPM2_RH_NULL,
-            &nvCmdAuths);
-    CheckPassed(rval);
-
-    /*
-     * And now read the data back.
-     * If the command is successful, the command
-     * HMAC was correct.
-     */
-    INIT_SIMPLE_TPM2B_SIZE( nvReadData );
-    sessionCmdRval = Tss2_Sys_NV_Read( simpleTestContext,
-            TPM20_INDEX_PASSWORD_TEST,
-            TPM20_INDEX_PASSWORD_TEST,
-            &nvCmdAuths, sizeof( dataToWrite ), 0,
-            &nvReadData, &nvRspAuths );
-    CheckPassed( sessionCmdRval );
-
-    /* Roll nonces for response */
-    roll_nonces(nvSession, &nvRspAuths.auths[0].nonce );
-
-    if (sessionCmdRval == TPM2_RC_SUCCESS) {
-        /*
-         * If the command was successful, check the
-         * response HMAC to make sure that the
-         * response was received correctly.
-         */
-        rval = check_response_hmac(
-                simpleTestContext,
-                &nvCmdAuths,
-                TPM20_INDEX_PASSWORD_TEST,
-                TPM20_INDEX_PASSWORD_TEST,
-                TPM2_RH_NULL,
-                &nvRspAuths);
-        CheckPassed(rval);
-    }
-
-    /* Check that write and read data are equal. */
-    if( memcmp( (void *)&nvReadData.buffer[0],
-            (void *)&nvWriteData.buffer[0], nvReadData.size ) )
-    {
-        LOG_ERROR("ERROR!! read data not equal to written data" );
-        Cleanup();
-    }
-
-    /*
-     * Now cleanup:  undefine the NV index and delete
-     * the NV index's entity table entry.
-     */
-
-    /* Setup authorization for undefining the NV index. */
-    nvCmdAuths.auths[0].sessionHandle = TPM2_RS_PW;
-    nvCmdAuths.auths[0].nonce.size = 0;
-    nvCmdAuths.auths[0].hmac.size = 0;
-
-    /* Undefine the NV index. */
-    rval = Tss2_Sys_NV_UndefineSpace( simpleTestContext,
-            TPM2_RH_PLATFORM, TPM20_INDEX_PASSWORD_TEST,
-            &nvCmdAuths, 0 );
-    CheckPassed( rval );
-
-    /* Delete the NV index's entry in the entity table. */
-    DeleteEntity(TPM20_INDEX_PASSWORD_TEST);
-
-    /* Remove the real session from sessions table. */
-    end_auth_session( nvSession );
-
-    sapi_teardown(simpleTestContext);
-}
-
 static void GetSetDecryptParamTests()
 {
     TPM2B_MAX_NV_BUFFER nvWriteData = {4, { 0xde, 0xad, 0xbe, 0xef,} };
@@ -2123,7 +1747,7 @@ static void GetSetDecryptParamTests()
     LOG_INFO("GET/SET DECRYPT PARAM TESTS:" );
 
     /* Create two sysContext structures. */
-    decryptParamTestSysContext = sapi_init_from_tcti_ctx(resMgrTctiContext);
+    decryptParamTestSysContext = sys_init_from_tcti_ctx(resMgrTctiContext);
     if (decryptParamTestSysContext == NULL)
         InitSysContextFailure();
 
@@ -2219,20 +1843,26 @@ static void GetSetDecryptParamTests()
     }
 
     /* Test for insufficient size. */
+    /* Create a buffer that is too large */
+    size_t testBufferSize = TPM2_MAX_COMMAND_SIZE -
+            BE_TO_HOST_32(((TPM20_Header_In *)(((_TSS2_SYS_CONTEXT_BLOB *)decryptParamTestSysContext)->cmdBuffer))->commandSize) + 1;
+    UINT8 testBuffer [testBufferSize];
+    memset(testBuffer, 0, testBufferSize);
+    memcpy(testBuffer, nvWriteData.buffer, nvWriteData.size);
+
     rval = Tss2_Sys_GetCpBuffer(decryptParamTestSysContext, &cpBufferUsedSize2, &cpBuffer2);
     CheckPassed(rval);
-    nvWriteData.size = TPM2_MAX_COMMAND_SIZE -
-            BE_TO_HOST_32(((TPM20_Header_In *)(((_TSS2_SYS_CONTEXT_BLOB *)decryptParamTestSysContext)->cmdBuffer))->commandSize) + 1;
-
-    rval = Tss2_Sys_SetDecryptParam(decryptParamTestSysContext, nvWriteData.size, nvWriteData.buffer);
+    rval = Tss2_Sys_SetDecryptParam(decryptParamTestSysContext, testBufferSize,
+            testBuffer);
     CheckFailed(rval, TSS2_SYS_RC_INSUFFICIENT_CONTEXT);
 
     /*
      * Test that one less will work.
      * This tests that we're checking the correct corner case.
      */
-    nvWriteData.size -= 1;
-    rval = Tss2_Sys_SetDecryptParam(decryptParamTestSysContext, nvWriteData.size, nvWriteData.buffer);
+    testBufferSize -= 1;
+    rval = Tss2_Sys_SetDecryptParam(decryptParamTestSysContext, testBufferSize,
+            testBuffer);
     CheckPassed(rval);
 
     rval = Tss2_Sys_NV_Write_Prepare( decryptParamTestSysContext, TPM20_INDEX_PASSWORD_TEST,
@@ -2273,7 +1903,7 @@ static void GetSetDecryptParamTests()
     rval = Tss2_Sys_SetDecryptParam( decryptParamTestSysContext, 1, &( nvWriteData.buffer[0] ) );
     CheckFailed( rval, TSS2_SYS_RC_BAD_SIZE );
 
-    sapi_teardown(decryptParamTestSysContext);
+    sys_teardown(decryptParamTestSysContext);
 }
 
 static void SysFinalizeTests()
@@ -2292,7 +1922,7 @@ static void GetContextSizeTests()
 
     LOG_INFO("SYS GETCONTEXTSIZE TESTS:" );
 
-    testSysContext = sapi_init_from_tcti_ctx(resMgrTctiContext);
+    testSysContext = sys_init_from_tcti_ctx(resMgrTctiContext);
     if (testSysContext == NULL)
         InitSysContextFailure();
 
@@ -2302,7 +1932,7 @@ static void GetContextSizeTests()
     rval = Tss2_Sys_GetTestResult_Prepare(testSysContext);
     CheckPassed(rval);
 
-    sapi_teardown(testSysContext);
+    sys_teardown(testSysContext);
 }
 
 static void GetTctiContextTests()
@@ -2313,7 +1943,7 @@ static void GetTctiContextTests()
 
     LOG_INFO("SYS GETTCTICONTEXT TESTS:" );
 
-    testSysContext = sapi_init_from_tcti_ctx(resMgrTctiContext);
+    testSysContext = sys_init_from_tcti_ctx(resMgrTctiContext);
     if (testSysContext == NULL)
         InitSysContextFailure();
 
@@ -2323,7 +1953,7 @@ static void GetTctiContextTests()
     rval = Tss2_Sys_GetTctiContext(0, &tctiContext);
     CheckFailed(rval, TSS2_SYS_RC_BAD_REFERENCE);
 
-    sapi_teardown(testSysContext);
+    sys_teardown(testSysContext);
 }
 
 static void GetSetEncryptParamTests()
@@ -2340,7 +1970,7 @@ static void GetSetEncryptParamTests()
 
     TSS2L_SYS_AUTH_RESPONSE sessionsDataOut;
     TSS2L_SYS_AUTH_COMMAND sessionsData = { .count = 1, .auths= {{
-        .sessionHandle = TPM2_RS_PW }}};
+        .sessionHandle = TPM2_RH_PW }}};
 
     TPM2B_MAX_NV_BUFFER nvReadData;
     const uint8_t       *cpBuffer;
@@ -2374,7 +2004,7 @@ static void GetSetEncryptParamTests()
     nvAttributes |= TPMA_NV_AUTHWRITE;
     nvAttributes |= TPMA_NV_PLATFORMCREATE;
 
-    rval = DefineNvIndex( TPM2_RH_PLATFORM, TPM2_RS_PW, &nvAuth, &authPolicy,
+    rval = DefineNvIndex( sysContext, TPM2_RH_PLATFORM, &nvAuth, &authPolicy,
             TPM20_INDEX_PASSWORD_TEST, TPM2_ALG_SHA1, nvAttributes, 32  );
     CheckPassed( rval ); /* #4 */
 
@@ -2505,7 +2135,7 @@ static void EcEphemeralTest()
 
     LOG_INFO("EC Ephemeral TESTS:" );
 
-    /* Test SAPI for case of Q size field not being set to 0. */
+    /* Test SYS for case of Q size field not being set to 0. */
     INIT_SIMPLE_TPM2B_SIZE( Q );
     rval = Tss2_Sys_EC_Ephemeral( sysContext, 0, TPM2_ECC_BN_P256, &Q, &counter, 0 );
     CheckFailed( rval, TSS2_SYS_RC_BAD_VALUE );
@@ -2516,29 +2146,31 @@ static void EcEphemeralTest()
 }
 
 int
-test_invoke (TSS2_SYS_CONTEXT *sapi_context)
+test_invoke (TSS2_SYS_CONTEXT *sys_context)
 {
     TSS2_RC rval = TSS2_RC_SUCCESS;
 
-    sysContext = sapi_context;
-    rval = Tss2_Sys_GetTctiContext (sapi_context, &resMgrTctiContext);
+#if !defined(TCTI_SWTPM) && !defined(TCTI_MSSIM)
+    /* SKIP */
+    return EXIT_SKIP;
+#endif
+
+    sysContext = sys_context;
+    rval = Tss2_Sys_GetTctiContext (sys_context, &resMgrTctiContext);
     if (rval != TSS2_RC_SUCCESS) {
-        printf ("Failed to get TCTI context from sapi_context: 0x%" PRIx32
+        printf ("Failed to get TCTI context from sys_context: 0x%" PRIx32
                 "\n", rval);
         return 1;
     }
 
-    nullSessionsData.auths[0].sessionHandle = TPM2_RS_PW;
+    nullSessionsData.auths[0].sessionHandle = TPM2_RH_PW;
     nullSessionsDataOut.count = 1;
     nullSessionsDataOut.auths[0].nonce = nullSessionNonceOut;
     nullSessionsDataOut.auths[0].hmac = nullSessionHmac;
     nullSessionNonceOut.size = 0;
     nullSessionNonce.size = 0;
 
-    rval = tcti_platform_command( resMgrTctiContext, MS_SIM_POWER_OFF );
-    CheckPassed(rval);
-
-    rval = tcti_platform_command( resMgrTctiContext, MS_SIM_POWER_ON );
+    rval = TpmReset();
     CheckPassed(rval);
 
     SysFinalizeTests();
@@ -2561,7 +2193,6 @@ test_invoke (TSS2_SYS_CONTEXT *sapi_context)
     TestDictionaryAttackLockReset();
     TestHierarchyControl();
     GetSetEncryptParamTests();
-    SimpleHmacOrPolicyTest( true );
     TestTpmGetCapability();
     TestPcrExtend();
     TestHash();
