@@ -31,111 +31,114 @@ static void
 check_hash_functions(void **state)
 {
     TSS2_RC rc;
-    IESYS_CRYPTO_CONTEXT_BLOB *context;
+    ESYS_CRYPTO_CONTEXT_BLOB *context;
     uint8_t buffer[10] = { 0 };
     TPM2B tpm2b;
     size_t size = 0;
 
-    rc = iesys_crypto_hash_start(NULL, TPM2_ALG_SHA384);
-    assert_int_equal (rc, TSS2_ESYS_RC_BAD_REFERENCE);
-
-#ifndef OSSL
-    rc = iesys_crypto_hash_start(&context, TPM2_ALG_SHA512);
-    assert_int_equal (rc, TSS2_ESYS_RC_NOT_IMPLEMENTED);
-#endif
-
-    rc = iesys_crypto_hash_start(&context, 0);
-    assert_int_equal (rc, TSS2_ESYS_RC_NOT_IMPLEMENTED);
-
-    rc = iesys_crypto_hash_start(&context, TPM2_ALG_SHA384);
+    ESYS_CRYPTO_CALLBACKS crypto_cb = { 0 };
+    rc = iesys_initialize_crypto_backend(&crypto_cb, NULL);
     assert_int_equal (rc, TSS2_RC_SUCCESS);
 
-    rc = iesys_crypto_hash_finish(NULL, &buffer[0], &size);
+    rc = iesys_crypto_hash_start(&crypto_cb, NULL, TPM2_ALG_SHA384);
     assert_int_equal (rc, TSS2_ESYS_RC_BAD_REFERENCE);
 
-    rc = iesys_crypto_hash_finish(&context, &buffer[0], &size);
+    rc = iesys_crypto_hash_start(&crypto_cb, &context, 0);
+    assert_int_equal (rc, TSS2_ESYS_RC_NOT_IMPLEMENTED);
+
+    rc = iesys_crypto_hash_start(&crypto_cb, &context, TPM2_ALG_SHA512);
+    assert_int_equal (rc, TSS2_RC_SUCCESS);
+
+    rc = iesys_crypto_hash_finish(&crypto_cb, NULL, &buffer[0], &size);
+    assert_int_equal (rc, TSS2_ESYS_RC_BAD_REFERENCE);
+
+    rc = iesys_crypto_hash_finish(&crypto_cb, &context, &buffer[0], &size);
     assert_int_equal (rc, TSS2_ESYS_RC_BAD_SIZE);
 
-    iesys_crypto_hash_abort(NULL);
-    iesys_crypto_hash_abort(&context);
+    iesys_crypto_hash_abort(&crypto_cb, NULL);
+    iesys_crypto_hash_abort(&crypto_cb, &context);
 
-    rc = iesys_crypto_hash_update(NULL, &buffer[0], 10);
+    rc = iesys_crypto_hash_update(&crypto_cb, NULL, &buffer[0], 10);
     assert_int_equal (rc, TSS2_ESYS_RC_BAD_REFERENCE);
 
-    rc = iesys_crypto_hash_update2b(NULL, &tpm2b);
+    rc = iesys_crypto_hash_update2b(&crypto_cb, NULL, &tpm2b);
     assert_int_equal (rc, TSS2_ESYS_RC_BAD_REFERENCE);
 
     /* Create invalid context */
-    rc = iesys_crypto_hmac_start(&context, TPM2_ALG_SHA1, &buffer[0], 10);
+    rc = iesys_crypto_hmac_start(&crypto_cb, &context, TPM2_ALG_SHA1, &buffer[0], 10);
     assert_int_equal (rc, TSS2_RC_SUCCESS);
 
-    iesys_crypto_hash_abort(&context);
+    iesys_crypto_hash_abort(&crypto_cb, &context);
 
-    rc = iesys_crypto_hash_update(context, &buffer[0], 10);
+    rc = iesys_crypto_hash_update(&crypto_cb, context, &buffer[0], 10);
     assert_int_equal (rc, TSS2_ESYS_RC_BAD_REFERENCE);
 
-    rc = iesys_crypto_hash_finish(&context, &buffer[0], &size);
+    rc = iesys_crypto_hash_finish(&crypto_cb, &context, &buffer[0], &size);
     assert_int_equal (rc, TSS2_ESYS_RC_BAD_REFERENCE);
 
     /* cleanup */
-    iesys_crypto_hmac_abort(&context);
+    iesys_crypto_hmac_abort(&crypto_cb, &context);
 }
 
 static void
 check_hmac_functions(void **state)
 {
     TSS2_RC rc;
-    IESYS_CRYPTO_CONTEXT_BLOB *context;
+    ESYS_CRYPTO_CONTEXT_BLOB *context;
     uint8_t buffer[10] = { 0 };
     TPM2B tpm2b;
     size_t size = 0;
 
-    rc = iesys_crypto_hmac_start(NULL, TPM2_ALG_SHA384, &buffer[0], 10);
+    ESYS_CRYPTO_CALLBACKS crypto_cb = { 0 };
+    rc = iesys_initialize_crypto_backend(&crypto_cb, NULL);
+    assert_int_equal (rc, TSS2_RC_SUCCESS);
+
+    rc = iesys_crypto_hmac_start(&crypto_cb, NULL, TPM2_ALG_SHA384, &buffer[0], 10);
     assert_int_equal (rc, TSS2_ESYS_RC_BAD_REFERENCE);
 
 #ifndef OSSL
-    rc = iesys_crypto_hmac_start(&context, TPM2_ALG_SHA512, &buffer[0], 10);
+    rc = iesys_crypto_hmac_start(&crypto_cb, &context, TPM2_ALG_SHA512, &buffer[0], 10);
     assert_int_equal (rc, TSS2_ESYS_RC_NOT_IMPLEMENTED);
 #endif
 
-    rc = iesys_crypto_hmac_start(&context, 0,  &buffer[0], 10);
+    rc = iesys_crypto_hmac_start(&crypto_cb, &context, 0,  &buffer[0], 10);
     assert_int_equal (rc, TSS2_ESYS_RC_NOT_IMPLEMENTED);
 
-    rc = iesys_crypto_hmac_start(&context, TPM2_ALG_SHA1,  &buffer[0], 10);
+    rc = iesys_crypto_hmac_start(&crypto_cb, &context, TPM2_ALG_SHA1,  &buffer[0], 10);
     assert_int_equal (rc, TSS2_RC_SUCCESS);
 
-    rc = iesys_crypto_hmac_finish(NULL, &buffer[0], &size);
+    rc = iesys_crypto_hmac_finish(&crypto_cb, NULL, &buffer[0], &size);
     assert_int_equal (rc, TSS2_ESYS_RC_BAD_REFERENCE);
 
-    rc = iesys_crypto_hmac_finish2b(NULL, &tpm2b);
+    rc = iesys_crypto_hmac_finish2b(&crypto_cb, NULL, &tpm2b);
     assert_int_equal (rc, TSS2_ESYS_RC_BAD_REFERENCE);
 
-    rc = iesys_crypto_hmac_finish(&context, &buffer[0], &size);
+    rc = iesys_crypto_hmac_finish(&crypto_cb, &context, &buffer[0], &size);
     assert_int_equal (rc, TSS2_ESYS_RC_BAD_SIZE);
 
-    iesys_crypto_hmac_abort(NULL);
-    iesys_crypto_hmac_abort(&context);
+    iesys_crypto_hmac_abort(&crypto_cb, NULL);
+    iesys_crypto_hmac_abort(&crypto_cb, &context);
 
-    rc = iesys_crypto_hmac_update(NULL, &buffer[0], 10);
+    rc = iesys_crypto_hmac_update(&crypto_cb, NULL, &buffer[0], 10);
     assert_int_equal (rc, TSS2_ESYS_RC_BAD_REFERENCE);
 
-    rc = iesys_crypto_hmac_update2b(NULL, &tpm2b);
+    rc = iesys_crypto_hmac_update2b(&crypto_cb, NULL, &tpm2b);
     assert_int_equal (rc, TSS2_ESYS_RC_BAD_REFERENCE);
 
     /* Create invalid context */
-    rc = iesys_crypto_hash_start(&context, TPM2_ALG_SHA1);
+    rc = iesys_crypto_hash_start(&crypto_cb, &context, TPM2_ALG_SHA1);
     assert_int_equal (rc, TSS2_RC_SUCCESS);
 
-    iesys_crypto_hmac_abort(&context);
+    iesys_crypto_hmac_abort(&crypto_cb, &context);
 
-    rc = iesys_crypto_hmac_update(context, &buffer[0], 10);
+    rc = iesys_crypto_hmac_update(&crypto_cb, context, &buffer[0], 10);
     assert_int_equal (rc, TSS2_ESYS_RC_BAD_REFERENCE);
 
-    rc = iesys_crypto_hmac_finish(&context, &buffer[0], &size);
+    rc = iesys_crypto_hmac_finish(&crypto_cb, &context, &buffer[0], &size);
     assert_int_equal (rc, TSS2_ESYS_RC_BAD_REFERENCE);
 
     /* cleanup */
-    iesys_crypto_hash_abort(&context);
+    iesys_crypto_hash_abort(&crypto_cb, &context);
 }
 
 static void
@@ -144,7 +147,12 @@ check_random(void **state)
     TSS2_RC rc;
     size_t num_bytes = 0;
     TPM2B_NONCE nonce;
-    rc = iesys_crypto_random2b(&nonce, num_bytes);
+
+    ESYS_CRYPTO_CALLBACKS crypto_cb = { 0 };
+    rc = iesys_initialize_crypto_backend(&crypto_cb, NULL);
+    assert_int_equal (rc, TSS2_RC_SUCCESS);
+
+    rc = iesys_crypto_get_random2b(&crypto_cb, &nonce, num_bytes);
     assert_int_equal (rc, TSS2_RC_SUCCESS);
 }
 
@@ -191,12 +199,17 @@ check_pk_encrypt(void **state)
     };
 
     inPublicRSA.publicArea.nameAlg = 0;
-    rc = iesys_crypto_pk_encrypt(&inPublicRSA, size, &in_buffer[0], size, &out_buffer[0], &size, "LABEL");
+
+    ESYS_CRYPTO_CALLBACKS crypto_cb = { 0 };
+    rc = iesys_initialize_crypto_backend(&crypto_cb, NULL);
+    assert_int_equal (rc, TSS2_RC_SUCCESS);
+
+    rc = iesys_crypto_rsa_pk_encrypt(&crypto_cb, &inPublicRSA, size, &in_buffer[0], size, &out_buffer[0], &size, "LABEL");
     assert_int_equal (rc, TSS2_ESYS_RC_NOT_IMPLEMENTED);
 
     inPublicRSA.publicArea.nameAlg = TPM2_ALG_SHA1;
     inPublicRSA.publicArea.parameters.rsaDetail.scheme.scheme = 0;
-    rc = iesys_crypto_pk_encrypt(&inPublicRSA, size, &in_buffer[0], size, &out_buffer[0], &size, "LABEL");
+    rc = iesys_crypto_rsa_pk_encrypt(&crypto_cb, &inPublicRSA, size, &in_buffer[0], size, &out_buffer[0], &size, "LABEL");
     assert_int_equal (rc, TSS2_ESYS_RC_BAD_VALUE);
 }
 
@@ -209,34 +222,38 @@ check_aes_encrypt(void **state)
     uint8_t buffer[5] = { 1, 2, 3, 4, 5 };
     size_t size = 5;
 
-    rc = iesys_crypto_sym_aes_encrypt(NULL, TPM2_ALG_AES, 192, TPM2_ALG_CFB,
+    ESYS_CRYPTO_CALLBACKS crypto_cb = { 0 };
+    rc = iesys_initialize_crypto_backend(&crypto_cb, NULL);
+    assert_int_equal (rc, TSS2_RC_SUCCESS);
+
+    rc = iesys_crypto_aes_encrypt(&crypto_cb, NULL, TPM2_ALG_AES, 192, TPM2_ALG_CFB,
                                       &buffer[0], size, &key[0]);
     assert_int_equal (rc, TSS2_ESYS_RC_BAD_REFERENCE);
 
-    rc = iesys_crypto_sym_aes_encrypt(&key[0], TPM2_ALG_AES, 192, TPM2_ALG_CFB,
+    rc = iesys_crypto_aes_encrypt(&crypto_cb, &key[0], TPM2_ALG_AES, 192, TPM2_ALG_CFB,
                                       &buffer[0], size, &key[0]);
     assert_int_equal (rc, TSS2_RC_SUCCESS);
-    rc = iesys_crypto_sym_aes_encrypt(&key[0], TPM2_ALG_AES, 256, TPM2_ALG_CFB,
+    rc = iesys_crypto_aes_encrypt(&crypto_cb, &key[0], TPM2_ALG_AES, 256, TPM2_ALG_CFB,
                                       &buffer[0], size, &key[0]);
     assert_int_equal (rc, TSS2_RC_SUCCESS);
 
-    rc = iesys_crypto_sym_aes_encrypt(&key[0], 0, 256, TPM2_ALG_CFB,
+    rc = iesys_crypto_aes_encrypt(&crypto_cb, &key[0], 0, 256, TPM2_ALG_CFB,
                                       &buffer[0], size, &key[0]);
     assert_int_equal (rc, TSS2_ESYS_RC_BAD_VALUE);
 
-    rc = iesys_crypto_sym_aes_encrypt(&key[0], TPM2_ALG_AES, 256, 0,
+    rc = iesys_crypto_aes_encrypt(&crypto_cb, &key[0], TPM2_ALG_AES, 256, 0,
                                       &buffer[0], size, &key[0]);
     assert_int_equal (rc, TSS2_ESYS_RC_BAD_VALUE);
 
-    rc = iesys_crypto_sym_aes_encrypt(&key[0], TPM2_ALG_AES, 999, TPM2_ALG_CFB,
+    rc = iesys_crypto_aes_encrypt(&crypto_cb, &key[0], TPM2_ALG_AES, 999, TPM2_ALG_CFB,
                                       &buffer[0], size, &key[0]);
     assert_int_equal (rc, TSS2_ESYS_RC_BAD_VALUE);
 
-    rc = iesys_crypto_sym_aes_decrypt(NULL, TPM2_ALG_AES, 192, TPM2_ALG_CFB,
+    rc = iesys_crypto_aes_decrypt(&crypto_cb, NULL, TPM2_ALG_AES, 192, TPM2_ALG_CFB,
                                       &buffer[0], size, &key[0]);
     assert_int_equal (rc, TSS2_ESYS_RC_BAD_REFERENCE);
 
-    rc = iesys_crypto_sym_aes_decrypt(&key[0], 0, 192, TPM2_ALG_CFB,
+    rc = iesys_crypto_aes_decrypt(&crypto_cb, &key[0], 0, 192, TPM2_ALG_CFB,
                                       &buffer[0], size, &key[0]);
     assert_int_equal (rc, TSS2_ESYS_RC_BAD_VALUE);
 }
@@ -275,6 +292,82 @@ check_get_sys_context(void **state)
     Esys_Finalize(&ctx);
 }
 
+#define CHECK_BACKEND_FN(backend, fn) \
+    assert_int_equal(backend.fn, _iesys_crypto_##fn)
+
+static TSS2_RC
+    crypto_init(void *userdata)
+{
+    assert_int_equal(userdata, (void *)0xDEADBEEF);
+    return 0x42;
+}
+
+static void test_backend_set(void **state) {
+
+    ESYS_CRYPTO_CALLBACKS crypto_cb = { 0 };
+    TSS2_RC rc = iesys_initialize_crypto_backend(&crypto_cb, NULL);
+    assert_int_equal (rc, TSS2_RC_SUCCESS);
+
+    CHECK_BACKEND_FN(crypto_cb, hash_start);
+    CHECK_BACKEND_FN(crypto_cb, hash_update);
+    CHECK_BACKEND_FN(crypto_cb, hash_finish);
+    CHECK_BACKEND_FN(crypto_cb, hash_abort);
+
+    CHECK_BACKEND_FN(crypto_cb, hmac_start);
+    CHECK_BACKEND_FN(crypto_cb, hmac_update);
+    CHECK_BACKEND_FN(crypto_cb, hmac_finish);
+    CHECK_BACKEND_FN(crypto_cb, hmac_abort);
+
+    CHECK_BACKEND_FN(crypto_cb, aes_decrypt);
+    CHECK_BACKEND_FN(crypto_cb, aes_encrypt);
+    CHECK_BACKEND_FN(crypto_cb, get_ecdh_point);
+    CHECK_BACKEND_FN(crypto_cb, get_random2b);
+    CHECK_BACKEND_FN(crypto_cb, rsa_pk_encrypt);
+
+    /* test a user change */
+    ESYS_CRYPTO_CALLBACKS user_cb = {
+        .aes_decrypt = (void *)0xBADCC0DE,
+        .aes_encrypt = (void *)0xBADCC0DE,
+        .get_ecdh_point = (void *)0xBADCC0DE,
+        .get_random2b = (void *)0xBADCC0DE,
+        .rsa_pk_encrypt = (void *)0xBADCC0DE,
+        .hash_abort = (void *)0xBADCC0DE,
+        .hash_finish = (void *)0xBADCC0DE,
+        .hash_start = (void *)0xBADCC0DE,
+        .hash_update = (void *)0xBADCC0DE,
+        .hmac_abort = (void *)0xBADCC0DE,
+        .hmac_finish = (void *)0xBADCC0DE,
+        .hmac_start = (void *)0xBADCC0DE,
+        .hmac_update = (void *)0xBADCC0DE,
+        .init = crypto_init,
+        .userdata = (void *)0xDEADBEEF
+    };
+
+    rc = iesys_initialize_crypto_backend(&crypto_cb, &user_cb);
+    assert_int_equal (rc, 0x42);
+    assert_memory_equal(&crypto_cb, &user_cb, sizeof(crypto_cb));
+
+    /* reset state */
+    rc = iesys_initialize_crypto_backend(&crypto_cb, NULL);
+    assert_int_equal (rc, TSS2_RC_SUCCESS);
+
+    CHECK_BACKEND_FN(crypto_cb, hash_start);
+    CHECK_BACKEND_FN(crypto_cb, hash_update);
+    CHECK_BACKEND_FN(crypto_cb, hash_finish);
+    CHECK_BACKEND_FN(crypto_cb, hash_abort);
+
+    CHECK_BACKEND_FN(crypto_cb, hmac_start);
+    CHECK_BACKEND_FN(crypto_cb, hmac_update);
+    CHECK_BACKEND_FN(crypto_cb, hmac_finish);
+    CHECK_BACKEND_FN(crypto_cb, hmac_abort);
+
+    CHECK_BACKEND_FN(crypto_cb, aes_decrypt);
+    CHECK_BACKEND_FN(crypto_cb, aes_encrypt);
+    CHECK_BACKEND_FN(crypto_cb, get_ecdh_point);
+    CHECK_BACKEND_FN(crypto_cb, get_random2b);
+    CHECK_BACKEND_FN(crypto_cb, rsa_pk_encrypt);
+}
+
 int
 main(int argc, char *argv[])
 {
@@ -286,6 +379,7 @@ main(int argc, char *argv[])
         cmocka_unit_test(check_aes_encrypt),
         cmocka_unit_test(check_free),
         cmocka_unit_test(check_get_sys_context),
+        cmocka_unit_test(test_backend_set)
     };
     return cmocka_run_group_tests(tests, NULL, NULL);
 }
